# -*- coding: UTF-8 -*-
# Code generated by lark suite oapi sdk gen

from typing import List, Dict, Any
from ....utils.dt import to_json_decorator
from ....event.model.event import *
import attr




@to_json_decorator
@attr.s
class TextRun(object):
    text = attr.ib(type=str, default=None, metadata={'json': 'text'})


@to_json_decorator
@attr.s
class Person(object):
    user_id = attr.ib(type=str, default=None, metadata={'json': 'user_id'})


@to_json_decorator
@attr.s
class DocsLink(object):
    url = attr.ib(type=str, default=None, metadata={'json': 'url'})


@to_json_decorator
@attr.s
class ReplyElement(object):
    type = attr.ib(type=str, default=None, metadata={'json': 'type'})
    text_run = attr.ib(type=TextRun, default=None, metadata={'json': 'text_run'})
    docs_link = attr.ib(type=DocsLink, default=None, metadata={'json': 'docs_link'})
    person = attr.ib(type=Person, default=None, metadata={'json': 'person'})


@to_json_decorator
@attr.s
class ReplyContent(object):
    elements = attr.ib(type=List[ReplyElement], default=None, metadata={'json': 'elements'})


@to_json_decorator
@attr.s
class FileCommentReply(object):
    reply_id = attr.ib(type=str, default=None, metadata={'json': 'reply_id'})
    user_id = attr.ib(type=str, default=None, metadata={'json': 'user_id'})
    create_time = attr.ib(type=int, default=None, metadata={'json': 'create_time'})
    update_time = attr.ib(type=int, default=None, metadata={'json': 'update_time'})
    content = attr.ib(type=ReplyContent, default=None, metadata={'json': 'content'})


@to_json_decorator
@attr.s
class ReplyList(object):
    replies = attr.ib(type=List[FileCommentReply], default=None, metadata={'json': 'replies'})


@to_json_decorator
@attr.s
class FileComment(object):
    comment_id = attr.ib(type=str, default=None, metadata={'json': 'comment_id'})
    user_id = attr.ib(type=str, default=None, metadata={'json': 'user_id'})
    create_time = attr.ib(type=int, default=None, metadata={'json': 'create_time'})
    update_time = attr.ib(type=int, default=None, metadata={'json': 'update_time'})
    is_solved = attr.ib(type=bool, default=None, metadata={'json': 'is_solved'})
    solved_time = attr.ib(type=int, default=None, metadata={'json': 'solved_time'})
    solver_user_id = attr.ib(type=str, default=None, metadata={'json': 'solver_user_id'})
    reply_list = attr.ib(type=ReplyList, default=None, metadata={'json': 'reply_list'})


@to_json_decorator
@attr.s
class UserId(object):
    user_id = attr.ib(type=str, default=None, metadata={'json': 'user_id'})
    open_id = attr.ib(type=str, default=None, metadata={'json': 'open_id'})
    union_id = attr.ib(type=str, default=None, metadata={'json': 'union_id'})


@to_json_decorator
@attr.s
class File(object):
    file_token = attr.ib(type=str, default=None, metadata={'json': 'file_token'})
    file_name = attr.ib(type=str, default=None, metadata={'json': 'file_name'})
    size = attr.ib(type=int, default=None, metadata={'json': 'size'})
    mime_type = attr.ib(type=str, default=None, metadata={'json': 'mime_type'})


@to_json_decorator
@attr.s
class Media(object):
    file_token = attr.ib(type=str, default=None, metadata={'json': 'file_token'})
    file_name = attr.ib(type=str, default=None, metadata={'json': 'file_name'})
    size = attr.ib(type=int, default=None, metadata={'json': 'size'})
    mime_type = attr.ib(type=str, default=None, metadata={'json': 'mime_type'})


@to_json_decorator
@attr.s
class TmpDownloadUrl(object):
    file_token = attr.ib(type=str, default=None, metadata={'json': 'file_token'})
    tmp_download_url = attr.ib(type=str, default=None, metadata={'json': 'tmp_download_url'})


@to_json_decorator
@attr.s
class UploadInfo(object):
    file_name = attr.ib(type=str, default=None, metadata={'json': 'file_name'})
    parent_type = attr.ib(type=str, default=None, metadata={'json': 'parent_type'})
    parent_node = attr.ib(type=str, default=None, metadata={'json': 'parent_node'})
    size = attr.ib(type=int, default=None, metadata={'json': 'size'})





@to_json_decorator
@attr.s
class FileUploadFinishReqBody(object):
    upload_id = attr.ib(type=str, default=None, metadata={'json': 'upload_id'})
    block_num = attr.ib(type=int, default=None, metadata={'json': 'block_num'})


@attr.s
class FileUploadFinishResult(object):
    file_token = attr.ib(type=str, default=None, metadata={'json': 'file_token'})



@attr.s
class FileUploadPrepareResult(object):
    upload_id = attr.ib(type=str, default=None, metadata={'json': 'upload_id'})
    block_size = attr.ib(type=int, default=None, metadata={'json': 'block_size'})
    block_num = attr.ib(type=int, default=None, metadata={'json': 'block_num'})



@attr.s
class MediaUploadAllResult(object):
    file_token = attr.ib(type=str, default=None, metadata={'json': 'file_token'})


@to_json_decorator
@attr.s
class MediaUploadFinishReqBody(object):
    upload_id = attr.ib(type=str, default=None, metadata={'json': 'upload_id'})
    block_num = attr.ib(type=int, default=None, metadata={'json': 'block_num'})


@attr.s
class MediaUploadFinishResult(object):
    file_token = attr.ib(type=str, default=None, metadata={'json': 'file_token'})



@attr.s
class FileUploadAllResult(object):
    file_token = attr.ib(type=str, default=None, metadata={'json': 'file_token'})



@attr.s
class MediaUploadPrepareResult(object):
    upload_id = attr.ib(type=str, default=None, metadata={'json': 'upload_id'})
    block_size = attr.ib(type=int, default=None, metadata={'json': 'block_size'})
    block_num = attr.ib(type=int, default=None, metadata={'json': 'block_num'})





@attr.s
class MediaBatchGetTmpDownloadUrlResult(object):
    tmp_download_urls = attr.ib(type=List[TmpDownloadUrl], default=None, metadata={'json': 'tmp_download_urls'})












@to_json_decorator
@attr.s
class FileCommentReplyUpdateReqBody(object):
    content = attr.ib(type=ReplyContent, default=None, metadata={'json': 'content'})




@attr.s
class FileCommentListResult(object):
    has_more = attr.ib(type=bool, default=None, metadata={'json': 'has_more'})
    page_token = attr.ib(type=str, default=None, metadata={'json': 'page_token'})
    items = attr.ib(type=List[FileComment], default=None, metadata={'json': 'items'})


@to_json_decorator
@attr.s
class FileCommentPatchReqBody(object):
    is_solved = attr.ib(type=bool, default=None, metadata={'json': 'is_solved'})





@attr.s
class FileDeletedEventData(object):
    file_type = attr.ib(type=str, default=None, metadata={'json': 'file_type'})
    file_token = attr.ib(type=str, default=None, metadata={'json': 'file_token'})
    operator_id = attr.ib(type=UserId, default=None, metadata={'json': 'operator_id'})


@attr.s
class FileDeletedEvent(BaseEventV2):
    event = attr.ib(type=FileDeletedEventData, default=None)



@attr.s
class FilePermissionMemberAddedEventData(object):
    file_type = attr.ib(type=str, default=None, metadata={'json': 'file_type'})
    file_token = attr.ib(type=str, default=None, metadata={'json': 'file_token'})
    operator_id = attr.ib(type=UserId, default=None, metadata={'json': 'operator_id'})
    user_list = attr.ib(type=List[UserId], default=None, metadata={'json': 'user_list'})
    chat_list = attr.ib(type=List[str], default=None, metadata={'json': 'chat_list'})


@attr.s
class FilePermissionMemberAddedEvent(BaseEventV2):
    event = attr.ib(type=FilePermissionMemberAddedEventData, default=None)



@attr.s
class FilePermissionMemberRemovedEventData(object):
    file_type = attr.ib(type=str, default=None, metadata={'json': 'file_type'})
    file_token = attr.ib(type=str, default=None, metadata={'json': 'file_token'})
    operator_id = attr.ib(type=UserId, default=None, metadata={'json': 'operator_id'})
    user_list = attr.ib(type=List[UserId], default=None, metadata={'json': 'user_list'})
    chat_list = attr.ib(type=List[str], default=None, metadata={'json': 'chat_list'})


@attr.s
class FilePermissionMemberRemovedEvent(BaseEventV2):
    event = attr.ib(type=FilePermissionMemberRemovedEventData, default=None)



@attr.s
class FileReadEventData(object):
    file_type = attr.ib(type=str, default=None, metadata={'json': 'file_type'})
    file_token = attr.ib(type=str, default=None, metadata={'json': 'file_token'})
    operator_id_list = attr.ib(type=List[UserId], default=None, metadata={'json': 'operator_id_list'})


@attr.s
class FileReadEvent(BaseEventV2):
    event = attr.ib(type=FileReadEventData, default=None)



@attr.s
class FileTitleUpdatedEventData(object):
    file_type = attr.ib(type=str, default=None, metadata={'json': 'file_type'})
    file_token = attr.ib(type=str, default=None, metadata={'json': 'file_token'})
    operator_id = attr.ib(type=UserId, default=None, metadata={'json': 'operator_id'})


@attr.s
class FileTitleUpdatedEvent(BaseEventV2):
    event = attr.ib(type=FileTitleUpdatedEventData, default=None)



@attr.s
class FileTrashedEventData(object):
    file_type = attr.ib(type=str, default=None, metadata={'json': 'file_type'})
    file_token = attr.ib(type=str, default=None, metadata={'json': 'file_token'})
    operator_id = attr.ib(type=UserId, default=None, metadata={'json': 'operator_id'})


@attr.s
class FileTrashedEvent(BaseEventV2):
    event = attr.ib(type=FileTrashedEventData, default=None)



# -*- coding: UTF-8 -*-
# Code generated by lark suite oapi sdk gen

from typing import List, Dict, Any
from ....utils.dt import to_json_decorator
from ....event.model.event import *
import attr




@to_json_decorator
@attr.s
class ReservePermissionChecker(object):
    check_field = attr.ib(type=int, default=None, metadata={'json': 'check_field'})
    check_mode = attr.ib(type=int, default=None, metadata={'json': 'check_mode'})
    check_list = attr.ib(type=List[str], default=None, metadata={'json': 'check_list'})


@to_json_decorator
@attr.s
class ReserveActionPermission(object):
    permission = attr.ib(type=int, default=None, metadata={'json': 'permission'})
    permission_checkers = attr.ib(type=List[ReservePermissionChecker], default=None, metadata={'json': 'permission_checkers'})


@to_json_decorator
@attr.s
class ReserveMeetingSetting(object):
    topic = attr.ib(type=str, default=None, metadata={'json': 'topic'})
    action_permissions = attr.ib(type=List[ReserveActionPermission], default=None, metadata={'json': 'action_permissions'})


@to_json_decorator
@attr.s
class Reserve(object):
    __int_to_string_fields__ = attr.ib(type=List[str], default=["id", "end_time"])
    id = attr.ib(type=int, default=None, metadata={'json': 'id'})
    meeting_no = attr.ib(type=str, default=None, metadata={'json': 'meeting_no'})
    url = attr.ib(type=str, default=None, metadata={'json': 'url'})
    app_link = attr.ib(type=str, default=None, metadata={'json': 'app_link'})
    end_time = attr.ib(type=int, default=None, metadata={'json': 'end_time'})
    expire_status = attr.ib(type=int, default=None, metadata={'json': 'expire_status'})
    reserve_user_id = attr.ib(type=str, default=None, metadata={'json': 'reserve_user_id'})
    meeting_settings = attr.ib(type=ReserveMeetingSetting, default=None, metadata={'json': 'meeting_settings'})


@to_json_decorator
@attr.s
class ReportMeetingDaily(object):
    __int_to_string_fields__ = attr.ib(type=List[str], default=["date", "meeting_count", "meeting_duration", "participant_count"])
    date = attr.ib(type=int, default=None, metadata={'json': 'date'})
    meeting_count = attr.ib(type=int, default=None, metadata={'json': 'meeting_count'})
    meeting_duration = attr.ib(type=int, default=None, metadata={'json': 'meeting_duration'})
    participant_count = attr.ib(type=int, default=None, metadata={'json': 'participant_count'})


@to_json_decorator
@attr.s
class Report(object):
    __int_to_string_fields__ = attr.ib(type=List[str], default=["total_meeting_count", "total_meeting_duration", "total_participant_count"])
    total_meeting_count = attr.ib(type=int, default=None, metadata={'json': 'total_meeting_count'})
    total_meeting_duration = attr.ib(type=int, default=None, metadata={'json': 'total_meeting_duration'})
    total_participant_count = attr.ib(type=int, default=None, metadata={'json': 'total_participant_count'})
    daily_report = attr.ib(type=List[ReportMeetingDaily], default=None, metadata={'json': 'daily_report'})


@to_json_decorator
@attr.s
class RoomDigitalSignageMaterial(object):
    id = attr.ib(type=str, default=None, metadata={'json': 'id'})
    name = attr.ib(type=str, default=None, metadata={'json': 'name'})
    material_type = attr.ib(type=int, default=None, metadata={'json': 'material_type'})
    url = attr.ib(type=str, default=None, metadata={'json': 'url'})
    duration = attr.ib(type=int, default=None, metadata={'json': 'duration'})
    cover = attr.ib(type=str, default=None, metadata={'json': 'cover'})
    md5 = attr.ib(type=str, default=None, metadata={'json': 'md5'})


@to_json_decorator
@attr.s
class RoomDigitalSignage(object):
    enable = attr.ib(type=bool, default=None, metadata={'json': 'enable'})
    mute = attr.ib(type=bool, default=None, metadata={'json': 'mute'})
    start_display = attr.ib(type=int, default=None, metadata={'json': 'start_display'})
    stop_display = attr.ib(type=int, default=None, metadata={'json': 'stop_display'})
    materials = attr.ib(type=List[RoomDigitalSignageMaterial], default=None, metadata={'json': 'materials'})


@to_json_decorator
@attr.s
class RoomConfig(object):
    room_background = attr.ib(type=str, default=None, metadata={'json': 'room_background'})
    display_background = attr.ib(type=str, default=None, metadata={'json': 'display_background'})
    digital_signage = attr.ib(type=RoomDigitalSignage, default=None, metadata={'json': 'digital_signage'})


@to_json_decorator
@attr.s
class MeetingUser(object):
    id = attr.ib(type=str, default=None, metadata={'json': 'id'})
    user_type = attr.ib(type=int, default=None, metadata={'json': 'user_type'})


@to_json_decorator
@attr.s
class MeetingParticipant(object):
    id = attr.ib(type=str, default=None, metadata={'json': 'id'})
    user_type = attr.ib(type=int, default=None, metadata={'json': 'user_type'})
    is_host = attr.ib(type=bool, default=None, metadata={'json': 'is_host'})
    is_cohost = attr.ib(type=bool, default=None, metadata={'json': 'is_cohost'})
    is_external = attr.ib(type=bool, default=None, metadata={'json': 'is_external'})
    status = attr.ib(type=int, default=None, metadata={'json': 'status'})


@to_json_decorator
@attr.s
class MeetingAbility(object):
    use_video = attr.ib(type=bool, default=None, metadata={'json': 'use_video'})
    use_audio = attr.ib(type=bool, default=None, metadata={'json': 'use_audio'})
    use_share_screen = attr.ib(type=bool, default=None, metadata={'json': 'use_share_screen'})
    use_follow_screen = attr.ib(type=bool, default=None, metadata={'json': 'use_follow_screen'})
    use_recording = attr.ib(type=bool, default=None, metadata={'json': 'use_recording'})
    use_pstn = attr.ib(type=bool, default=None, metadata={'json': 'use_pstn'})


@to_json_decorator
@attr.s
class Meeting(object):
    __int_to_string_fields__ = attr.ib(type=List[str], default=["id", "create_time", "start_time", "end_time", "participant_count"])
    id = attr.ib(type=int, default=None, metadata={'json': 'id'})
    topic = attr.ib(type=str, default=None, metadata={'json': 'topic'})
    url = attr.ib(type=str, default=None, metadata={'json': 'url'})
    create_time = attr.ib(type=int, default=None, metadata={'json': 'create_time'})
    start_time = attr.ib(type=int, default=None, metadata={'json': 'start_time'})
    end_time = attr.ib(type=int, default=None, metadata={'json': 'end_time'})
    host_user = attr.ib(type=MeetingUser, default=None, metadata={'json': 'host_user'})
    status = attr.ib(type=int, default=None, metadata={'json': 'status'})
    participant_count = attr.ib(type=int, default=None, metadata={'json': 'participant_count'})
    participants = attr.ib(type=List[MeetingParticipant], default=None, metadata={'json': 'participants'})
    ability = attr.ib(type=MeetingAbility, default=None, metadata={'json': 'ability'})


@to_json_decorator
@attr.s
class UserId(object):
    user_id = attr.ib(type=str, default=None, metadata={'json': 'user_id'})
    open_id = attr.ib(type=str, default=None, metadata={'json': 'open_id'})
    union_id = attr.ib(type=str, default=None, metadata={'json': 'union_id'})


@to_json_decorator
@attr.s
class MeetingEventUser(object):
    id = attr.ib(type=UserId, default=None, metadata={'json': 'id'})
    user_role = attr.ib(type=int, default=None, metadata={'json': 'user_role'})
    user_type = attr.ib(type=int, default=None, metadata={'json': 'user_type'})


@to_json_decorator
@attr.s
class MeetingEventMeeting(object):
    __int_to_string_fields__ = attr.ib(type=List[str], default=["id", "start_time", "end_time"])
    id = attr.ib(type=int, default=None, metadata={'json': 'id'})
    topic = attr.ib(type=str, default=None, metadata={'json': 'topic'})
    meeting_no = attr.ib(type=str, default=None, metadata={'json': 'meeting_no'})
    start_time = attr.ib(type=int, default=None, metadata={'json': 'start_time'})
    end_time = attr.ib(type=int, default=None, metadata={'json': 'end_time'})
    host_user = attr.ib(type=MeetingEventUser, default=None, metadata={'json': 'host_user'})
    owner = attr.ib(type=MeetingEventUser, default=None, metadata={'json': 'owner'})


@to_json_decorator
@attr.s
class MeetingParticipantResult(object):
    id = attr.ib(type=str, default=None, metadata={'json': 'id'})
    user_type = attr.ib(type=int, default=None, metadata={'json': 'user_type'})
    result = attr.ib(type=int, default=None, metadata={'json': 'result'})


@to_json_decorator
@attr.s
class RecordingPermissionObject(object):
    id = attr.ib(type=str, default=None, metadata={'json': 'id'})
    type = attr.ib(type=int, default=None, metadata={'json': 'type'})
    permission = attr.ib(type=int, default=None, metadata={'json': 'permission'})


@to_json_decorator
@attr.s
class ReportTopUser(object):
    __int_to_string_fields__ = attr.ib(type=List[str], default=["id", "meeting_count", "meeting_duration"])
    id = attr.ib(type=int, default=None, metadata={'json': 'id'})
    name = attr.ib(type=str, default=None, metadata={'json': 'name'})
    user_type = attr.ib(type=int, default=None, metadata={'json': 'user_type'})
    meeting_count = attr.ib(type=int, default=None, metadata={'json': 'meeting_count'})
    meeting_duration = attr.ib(type=int, default=None, metadata={'json': 'meeting_duration'})


@to_json_decorator
@attr.s
class MeetingInviteStatus(object):
    id = attr.ib(type=str, default=None, metadata={'json': 'id'})
    user_type = attr.ib(type=int, default=None, metadata={'json': 'user_type'})
    status = attr.ib(type=int, default=None, metadata={'json': 'status'})


@to_json_decorator
@attr.s
class MeetingRecording(object):
    __int_to_string_fields__ = attr.ib(type=List[str], default=["id", "meeting_id", "duration"])
    id = attr.ib(type=int, default=None, metadata={'json': 'id'})
    meeting_id = attr.ib(type=int, default=None, metadata={'json': 'meeting_id'})
    url = attr.ib(type=str, default=None, metadata={'json': 'url'})
    duration = attr.ib(type=int, default=None, metadata={'json': 'duration'})





@to_json_decorator
@attr.s
class MeetingInviteReqBody(object):
    invitees = attr.ib(type=List[MeetingUser], default=None, metadata={'json': 'invitees'})


@attr.s
class MeetingInviteResult(object):
    invite_results = attr.ib(type=List[MeetingInviteStatus], default=None, metadata={'json': 'invite_results'})



@attr.s
class MeetingListResult(object):
    has_more = attr.ib(type=bool, default=None, metadata={'json': 'has_more'})
    page_token = attr.ib(type=str, default=None, metadata={'json': 'page_token'})
    meetings = attr.ib(type=List[Meeting], default=None, metadata={'json': 'meetings'})



@attr.s
class ReportGetTopUserResult(object):
    top_user_report = attr.ib(type=List[ReportTopUser], default=None, metadata={'json': 'top_user_report'})


@to_json_decorator
@attr.s
class MeetingSetHostReqBody(object):
    host_user = attr.ib(type=MeetingUser, default=None, metadata={'json': 'host_user'})
    old_host_user = attr.ib(type=MeetingUser, default=None, metadata={'json': 'old_host_user'})


@attr.s
class MeetingSetHostResult(object):
    host_user = attr.ib(type=MeetingUser, default=None, metadata={'json': 'host_user'})



@attr.s
class MeetingRecordingGetResult(object):
    recording = attr.ib(type=MeetingRecording, default=None, metadata={'json': 'recording'})







@attr.s
class ReportGetDailyResult(object):
    meeting_report = attr.ib(type=Report, default=None, metadata={'json': 'meeting_report'})



@attr.s
class MeetingGetResult(object):
    meeting = attr.ib(type=Meeting, default=None, metadata={'json': 'meeting'})


@to_json_decorator
@attr.s
class RoomConfigSetReqBody(object):
    __int_to_string_fields__ = attr.ib(type=List[str], default=["country_id", "district_id", "building_id", "room_id"])
    scope = attr.ib(type=int, default=None, metadata={'json': 'scope'})
    country_id = attr.ib(type=int, default=None, metadata={'json': 'country_id'})
    district_id = attr.ib(type=int, default=None, metadata={'json': 'district_id'})
    building_id = attr.ib(type=int, default=None, metadata={'json': 'building_id'})
    floor_name = attr.ib(type=str, default=None, metadata={'json': 'floor_name'})
    room_id = attr.ib(type=int, default=None, metadata={'json': 'room_id'})
    room_config = attr.ib(type=RoomConfig, default=None, metadata={'json': 'room_config'})



@to_json_decorator
@attr.s
class MeetingRecordingSetPermissionReqBody(object):
    permission_objects = attr.ib(type=List[RecordingPermissionObject], default=None, metadata={'json': 'permission_objects'})



@to_json_decorator
@attr.s
class MeetingRecordingStartReqBody(object):
    timezone = attr.ib(type=int, default=None, metadata={'json': 'timezone'})



@to_json_decorator
@attr.s
class ReserveUpdateReqBody(object):
    __int_to_string_fields__ = attr.ib(type=List[str], default=["end_time"])
    end_time = attr.ib(type=int, default=None, metadata={'json': 'end_time'})
    meeting_settings = attr.ib(type=ReserveMeetingSetting, default=None, metadata={'json': 'meeting_settings'})


@attr.s
class ReserveUpdateResult(object):
    reserve = attr.ib(type=Reserve, default=None, metadata={'json': 'reserve'})


@to_json_decorator
@attr.s
class ReserveApplyReqBody(object):
    __int_to_string_fields__ = attr.ib(type=List[str], default=["end_time"])
    end_time = attr.ib(type=int, default=None, metadata={'json': 'end_time'})
    meeting_settings = attr.ib(type=ReserveMeetingSetting, default=None, metadata={'json': 'meeting_settings'})


@attr.s
class ReserveApplyResult(object):
    reserve = attr.ib(type=Reserve, default=None, metadata={'json': 'reserve'})



@attr.s
class ReserveGetResult(object):
    reserve = attr.ib(type=Reserve, default=None, metadata={'json': 'reserve'})



@attr.s
class ReserveGetActiveMeetingResult(object):
    meeting = attr.ib(type=Meeting, default=None, metadata={'json': 'meeting'})




@attr.s
class MeetingLeaveMeetingEventData(object):
    meeting = attr.ib(type=MeetingEventMeeting, default=None, metadata={'json': 'meeting'})
    operator = attr.ib(type=MeetingEventUser, default=None, metadata={'json': 'operator'})
    leave_reason = attr.ib(type=int, default=None, metadata={'json': 'leave_reason'})


@attr.s
class MeetingLeaveMeetingEvent(BaseEventV2):
    event = attr.ib(type=MeetingLeaveMeetingEventData, default=None)



@attr.s
class MeetingMeetingEndedEventData(object):
    meeting = attr.ib(type=MeetingEventMeeting, default=None, metadata={'json': 'meeting'})
    operator = attr.ib(type=MeetingEventUser, default=None, metadata={'json': 'operator'})


@attr.s
class MeetingMeetingEndedEvent(BaseEventV2):
    event = attr.ib(type=MeetingMeetingEndedEventData, default=None)



@attr.s
class MeetingMeetingStartedEventData(object):
    meeting = attr.ib(type=MeetingEventMeeting, default=None, metadata={'json': 'meeting'})
    operator = attr.ib(type=MeetingEventUser, default=None, metadata={'json': 'operator'})


@attr.s
class MeetingMeetingStartedEvent(BaseEventV2):
    event = attr.ib(type=MeetingMeetingStartedEventData, default=None)



@attr.s
class MeetingRecordingEndedEventData(object):
    meeting = attr.ib(type=MeetingEventMeeting, default=None, metadata={'json': 'meeting'})
    operator = attr.ib(type=MeetingEventUser, default=None, metadata={'json': 'operator'})


@attr.s
class MeetingRecordingEndedEvent(BaseEventV2):
    event = attr.ib(type=MeetingRecordingEndedEventData, default=None)



@attr.s
class MeetingShareEndedEventData(object):
    meeting = attr.ib(type=MeetingEventMeeting, default=None, metadata={'json': 'meeting'})
    operator = attr.ib(type=MeetingEventUser, default=None, metadata={'json': 'operator'})


@attr.s
class MeetingShareEndedEvent(BaseEventV2):
    event = attr.ib(type=MeetingShareEndedEventData, default=None)



@attr.s
class MeetingJoinMeetingEventData(object):
    meeting = attr.ib(type=MeetingEventMeeting, default=None, metadata={'json': 'meeting'})
    operator = attr.ib(type=MeetingEventUser, default=None, metadata={'json': 'operator'})


@attr.s
class MeetingJoinMeetingEvent(BaseEventV2):
    event = attr.ib(type=MeetingJoinMeetingEventData, default=None)



@attr.s
class MeetingRecordingStartedEventData(object):
    meeting = attr.ib(type=MeetingEventMeeting, default=None, metadata={'json': 'meeting'})
    operator = attr.ib(type=MeetingEventUser, default=None, metadata={'json': 'operator'})


@attr.s
class MeetingRecordingStartedEvent(BaseEventV2):
    event = attr.ib(type=MeetingRecordingStartedEventData, default=None)



@attr.s
class MeetingSendMeetingImEventData(object):
    meeting = attr.ib(type=MeetingEventMeeting, default=None, metadata={'json': 'meeting'})
    operator = attr.ib(type=MeetingEventUser, default=None, metadata={'json': 'operator'})
    content = attr.ib(type=str, default=None, metadata={'json': 'content'})


@attr.s
class MeetingSendMeetingImEvent(BaseEventV2):
    event = attr.ib(type=MeetingSendMeetingImEventData, default=None)



@attr.s
class MeetingShareStartedEventData(object):
    meeting = attr.ib(type=MeetingEventMeeting, default=None, metadata={'json': 'meeting'})
    operator = attr.ib(type=MeetingEventUser, default=None, metadata={'json': 'operator'})


@attr.s
class MeetingShareStartedEvent(BaseEventV2):
    event = attr.ib(type=MeetingShareStartedEventData, default=None)



@attr.s
class MeetingRecordingReadyEventData(object):
    meeting = attr.ib(type=MeetingEventMeeting, default=None, metadata={'json': 'meeting'})
    url = attr.ib(type=str, default=None, metadata={'json': 'url'})
    duration = attr.ib(type=int, default=None, metadata={'json': 'duration'})


@attr.s
class MeetingRecordingReadyEvent(BaseEventV2):
    event = attr.ib(type=MeetingRecordingReadyEventData, default=None)



# -*- coding: UTF-8 -*-
import json
from hashlib import sha1
from typing import Any, Union

from ...utils import make_datatype
from ...config import Config
from ...consts import *
from ...context import Context
from ..model.card import Challenge, Header, HTTPCard, Card
from ..model.error import *
from .manager import Manager

LARK_REQUEST_TIMESTAMP = "X-Lark-Request-Timestamp"
LARK_REQUEST_REQUEST_NONCE = "X-Lark-Request-Nonce"
LARK_SIGNATURE = "X-Lark-Signature"
LARK_REFRESH_TOKEN = "X-Refresh-Token"
CALLBACK_TYPE_CHALLENGE = "url_verification"
RESPONSE_FORMAT = '{"codemsg":"%s"}'
CHALLENGE_RESPONSE_FORMAT = '{"challenge":"%s"}'

HTTP_STATUS_OK = 200
HTTP_STATUS_INTERNAL_SERVER_ERROR = 500


class CardHandlers(object):

    def __init__(self, conf, httpCard):
        # type: (Config, HTTPCard) -> None
        self.conf = conf
        self.header = Header()
        self.httpCard = httpCard
        self.respBody = None  # type: Union[None, Any]
        self.ctx = Context()

    def init(self):  # type: () -> None
        req, ctx = self.httpCard.request, self.ctx
        __headers = req.header
        ctx.set(HTTP_HEADER_KEY, __headers)
        time_stamp = __headers.first_value(LARK_REQUEST_TIMESTAMP)
        nonce = __headers.first_value(LARK_REQUEST_REQUEST_NONCE)
        signature = __headers.first_value(LARK_SIGNATURE)
        refresh_token = __headers.first_value(LARK_REFRESH_TOKEN)
        self.header.time_stamp = time_stamp
        self.header.nonce = nonce
        self.header.signature = signature
        self.header.refresh_token = refresh_token

    def validate(self):  # type: () -> None
        httpCard, conf = self.httpCard, self.conf
        if not self.header.signature:
            return
        settings = conf.app_settings
        body = httpCard.request.body.decode('utf-8')
        signature = CardHandlers.__signature(self.header.nonce, self.header.time_stamp, body,
                                             settings.verification_token)
        if self.header.signature != signature:
            raise SignatureError('signature error')

    @staticmethod
    def __signature(nonce, timestamp, body, token):  # type: (str, str, str, str) -> str
        buf = '%s%s%s%s' % (timestamp, nonce, token, body)
        sha1_obj = sha1()
        sha1_obj.update(buf.encode('utf-8'))
        return sha1_obj.hexdigest()

    def parse(self):  # type: () -> None
        httpCard, conf = self.httpCard, self.conf
        req = httpCard.request
        req.body = json.loads(req.body)
        challenge = make_datatype(Challenge, req.body)
        httpCard.type = challenge.type
        httpCard.challenge = challenge.challenge
        if httpCard.type == CALLBACK_TYPE_CHALLENGE \
                and conf.app_settings.verification_token != challenge.token:
            raise TokenInvalidError(
                'AppSettings.verificationToken not equal token(%s)' % challenge.token)

    def handle(self):  # type: () -> None
        ctx, conf, httpCard = self.ctx, self.conf, self.httpCard
        if httpCard.type == CALLBACK_TYPE_CHALLENGE:
            return

        h = Manager.get_callback(conf)
        if h is None:
            raise HandlerNotFoundError('handler not found')
        card = make_datatype(Card, self.httpCard.request.body)
        self.respBody = h(ctx, conf, card)

    def complement(self, exception):  # type: (Union[None, Exception]) -> None
        card, conf = self.httpCard, self.conf

        if exception is not None:
            if isinstance(exception, HandlerNotFoundError):
                status_code = HTTP_STATUS_INTERNAL_SERVER_ERROR
                content_type = DEFAULT_CONTENT_TYPE
                body = RESPONSE_FORMAT % exception.__str__()
                err_msg = 'handler not found, %s' % exception.__str__()
            else:
                status_code = HTTP_STATUS_INTERNAL_SERVER_ERROR
                content_type = DEFAULT_CONTENT_TYPE
                body = RESPONSE_FORMAT % exception.__str__()
                err_msg = 'err occurred, %s' % exception.__str__()

            self.write_response(status_code, content_type, body)
            conf.logger.error('request_id=%s err_msg=%s' % (self.ctx.get_request_id(), err_msg))
            return

        if card.type == CALLBACK_TYPE_CHALLENGE:
            self.write_response(
                HTTP_STATUS_OK, DEFAULT_CONTENT_TYPE,
                CHALLENGE_RESPONSE_FORMAT % card.challenge)
            return

        if self.respBody is not None:
            if isinstance(self.respBody, dict):
                body = json.dumps(self.respBody)
            else:
                body = self.respBody
            self.write_response(HTTP_STATUS_OK, DEFAULT_CONTENT_TYPE, body)
        else:
            self.write_response(HTTP_STATUS_OK, DEFAULT_CONTENT_TYPE, RESPONSE_FORMAT % 'success')

    def handler(self):  # type: () -> None
        try:
            self.init()
            self.validate()
            self.parse()
            self.handle()
        except Exception as e:
            self.complement(e)
        else:
            self.complement(None)

    def write_response(self, status_code, content_type, body):
        # type: (int, str, Union[str, bytes]) -> None
        resp = self.httpCard.response
        if isinstance(body, str):
            body = body.encode('utf-8')
        resp.status_code = status_code
        resp.content_type = content_type
        resp.body = body



# -*- coding: UTF-8 -*-

class HandlerNotFoundError(RuntimeError):
    def __init__(self, *args):  # type: (object) -> None
        super(HandlerNotFoundError, self).__init__(args)


class SignatureError(RuntimeError):
    def __init__(self, *args):  # type: (object) -> None
        super(SignatureError, self).__init__(args)


class TokenInvalidError(RuntimeError):
    def __init__(self, *args):  # type: (object) -> None
        super(TokenInvalidError, self).__init__(args)



# -*- coding: UTF-8 -*-
# Code generated by lark suite oapi sdk gen

from typing import List, Dict, Any
from ....utils.dt import to_json_decorator
import attr




@to_json_decorator
@attr.s
class AuditWebContext(object):
    user_agent = attr.ib(type=str, default=None, metadata={'json': 'user_agent'})
    i_p = attr.ib(type=str, default=None, metadata={'json': 'IP'})


@to_json_decorator
@attr.s
class AuditRecipientDetail(object):
    permission_action_type = attr.ib(type=str, default=None, metadata={'json': 'permission_action_type'})


@to_json_decorator
@attr.s
class AuditRecipientEntity(object):
    recipient_type = attr.ib(type=str, default=None, metadata={'json': 'recipient_type'})
    recipient_value = attr.ib(type=str, default=None, metadata={'json': 'recipient_value'})
    recipient_detail = attr.ib(type=AuditRecipientDetail, default=None, metadata={'json': 'recipient_detail'})


@to_json_decorator
@attr.s
class AuditPcContext(object):
    udid = attr.ib(type=str, default=None, metadata={'json': 'udid'})
    did = attr.ib(type=str, default=None, metadata={'json': 'did'})
    app_ver = attr.ib(type=str, default=None, metadata={'json': 'app_ver'})
    ver = attr.ib(type=str, default=None, metadata={'json': 'ver'})
    os = attr.ib(type=str, default=None, metadata={'json': 'os'})
    wifip = attr.ib(type=str, default=None, metadata={'json': 'wifip'})
    region = attr.ib(type=str, default=None, metadata={'json': 'region'})
    i_p = attr.ib(type=str, default=None, metadata={'json': 'IP'})


@to_json_decorator
@attr.s
class AuditObjectDetail(object):
    clone_source = attr.ib(type=str, default=None, metadata={'json': 'clone_source'})
    text_detail = attr.ib(type=str, default=None, metadata={'json': 'text_detail'})
    file_name = attr.ib(type=str, default=None, metadata={'json': 'file_name'})
    third_party_app_i_d = attr.ib(type=str, default=None, metadata={'json': 'third_party_appID'})
    contain_file_num = attr.ib(type=int, default=None, metadata={'json': 'contain_file_num'})
    permission_setting_type = attr.ib(type=str, default=None, metadata={'json': 'permission_setting_type'})
    permission_external_access__type = attr.ib(type=bool, default=None, metadata={'json': 'permission_external_access_Type'})
    permission_share_type = attr.ib(type=str, default=None, metadata={'json': 'permission_share_type'})
    file_service_source = attr.ib(type=str, default=None, metadata={'json': 'file_service_source'})
    okr_download_content = attr.ib(type=str, default=None, metadata={'json': 'okr_download_content'})


@to_json_decorator
@attr.s
class AuditObjectEntity(object):
    object_type = attr.ib(type=str, default=None, metadata={'json': 'object_type'})
    object_value = attr.ib(type=str, default=None, metadata={'json': 'object_value'})
    object_name = attr.ib(type=str, default=None, metadata={'json': 'object_name'})
    object_owner = attr.ib(type=str, default=None, metadata={'json': 'object_owner'})
    object_detail = attr.ib(type=AuditObjectDetail, default=None, metadata={'json': 'object_detail'})


@to_json_decorator
@attr.s
class AuditIosContext(object):
    udid = attr.ib(type=str, default=None, metadata={'json': 'udid'})
    did = attr.ib(type=str, default=None, metadata={'json': 'did'})
    app_ver = attr.ib(type=str, default=None, metadata={'json': 'app_ver'})
    ver = attr.ib(type=str, default=None, metadata={'json': 'ver'})
    os = attr.ib(type=str, default=None, metadata={'json': 'os'})
    s_t_zone = attr.ib(type=str, default=None, metadata={'json': 'STZone'})
    m_l = attr.ib(type=str, default=None, metadata={'json': 'ML'})
    sjd = attr.ib(type=str, default=None, metadata={'json': 'sjd'})
    proxyip = attr.ib(type=str, default=None, metadata={'json': 'proxyip'})
    wifip = attr.ib(type=str, default=None, metadata={'json': 'wifip'})
    location = attr.ib(type=str, default=None, metadata={'json': 'location'})
    active_ip = attr.ib(type=str, default=None, metadata={'json': 'active_ip'})
    active_ip_detail = attr.ib(type=str, default=None, metadata={'json': 'active_ip_detail'})
    cell_base_station = attr.ib(type=str, default=None, metadata={'json': 'cell_base_station'})
    i_p = attr.ib(type=str, default=None, metadata={'json': 'IP'})


@to_json_decorator
@attr.s
class AuditEventExtend(object):
    comment_type = attr.ib(type=str, default=None, metadata={'json': 'comment_type'})
    app_detail = attr.ib(type=str, default=None, metadata={'json': 'app_detail'})
    two_step_validation = attr.ib(type=bool, default=None, metadata={'json': 'two_step_validation'})
    login_method = attr.ib(type=str, default=None, metadata={'json': 'login_method'})
    new_people_num_in_video = attr.ib(type=int, default=None, metadata={'json': 'new_people_num_in_video'})
    external_people_num_in_video = attr.ib(type=int, default=None, metadata={'json': 'external_people_num_in_video'})
    external_people_num_in_chat = attr.ib(type=int, default=None, metadata={'json': 'external_people_num_in_chat'})
    join_group = attr.ib(type=int, default=None, metadata={'json': 'join_group'})
    quit_group = attr.ib(type=int, default=None, metadata={'json': 'quit_group'})
    external_people_num_in_doc_share = attr.ib(type=int, default=None, metadata={'json': 'external_people_num_in_doc_share'})


@to_json_decorator
@attr.s
class AuditAndroidContext(object):
    udid = attr.ib(type=str, default=None, metadata={'json': 'udid'})
    did = attr.ib(type=str, default=None, metadata={'json': 'did'})
    app_ver = attr.ib(type=str, default=None, metadata={'json': 'app_ver'})
    ver = attr.ib(type=str, default=None, metadata={'json': 'ver'})
    region = attr.ib(type=str, default=None, metadata={'json': 'region'})
    id_i = attr.ib(type=str, default=None, metadata={'json': 'id_i'})
    id_r = attr.ib(type=str, default=None, metadata={'json': 'id_r'})
    hw_brand = attr.ib(type=str, default=None, metadata={'json': 'hw_brand'})
    hw_manuf = attr.ib(type=str, default=None, metadata={'json': 'hw_manuf'})
    wifip = attr.ib(type=str, default=None, metadata={'json': 'wifip'})
    route_iip = attr.ib(type=str, default=None, metadata={'json': 'route_iip'})
    route_gip = attr.ib(type=str, default=None, metadata={'json': 'route_gip'})
    env_su = attr.ib(type=str, default=None, metadata={'json': 'env_su'})
    env_tz = attr.ib(type=str, default=None, metadata={'json': 'env_tz'})
    env_ml = attr.ib(type=str, default=None, metadata={'json': 'env_ml'})
    location = attr.ib(type=str, default=None, metadata={'json': 'location'})
    active_ip = attr.ib(type=str, default=None, metadata={'json': 'active_ip'})
    active_ip_detail = attr.ib(type=str, default=None, metadata={'json': 'active_ip_detail'})
    cell_base_station = attr.ib(type=str, default=None, metadata={'json': 'cell_base_station'})
    i_p = attr.ib(type=str, default=None, metadata={'json': 'IP'})


@to_json_decorator
@attr.s
class AuditContext(object):
    terminal_type = attr.ib(type=int, default=None, metadata={'json': 'terminal_type'})
    ios_context = attr.ib(type=AuditIosContext, default=None, metadata={'json': 'ios_context'})
    pc_context = attr.ib(type=AuditPcContext, default=None, metadata={'json': 'pc_context'})
    web_context = attr.ib(type=AuditWebContext, default=None, metadata={'json': 'web_context'})
    android_context = attr.ib(type=AuditAndroidContext, default=None, metadata={'json': 'android_context'})


@to_json_decorator
@attr.s
class AuditInfo(object):
    event_id = attr.ib(type=str, default=None, metadata={'json': 'event_id'})
    event_name = attr.ib(type=str, default=None, metadata={'json': 'event_name'})
    department_ids = attr.ib(type=List[str], default=None, metadata={'json': 'department_ids'})
    event_module = attr.ib(type=int, default=None, metadata={'json': 'event_module'})
    operator_type = attr.ib(type=int, default=None, metadata={'json': 'operator_type'})
    operator_value = attr.ib(type=str, default=None, metadata={'json': 'operator_value'})
    objects = attr.ib(type=List[AuditObjectEntity], default=None, metadata={'json': 'objects'})
    recipients = attr.ib(type=List[AuditRecipientEntity], default=None, metadata={'json': 'recipients'})
    event_time = attr.ib(type=int, default=None, metadata={'json': 'event_time'})
    ip = attr.ib(type=str, default=None, metadata={'json': 'ip'})
    operator_app = attr.ib(type=str, default=None, metadata={'json': 'operator_app'})
    audit_context = attr.ib(type=AuditContext, default=None, metadata={'json': 'audit_context'})
    extend = attr.ib(type=AuditEventExtend, default=None, metadata={'json': 'extend'})
    operator_app_name = attr.ib(type=str, default=None, metadata={'json': 'operator_app_name'})


@to_json_decorator
@attr.s
class AdminDeptStat(object):
    date = attr.ib(type=str, default=None, metadata={'json': 'date'})
    department_id = attr.ib(type=str, default=None, metadata={'json': 'department_id'})
    department_name = attr.ib(type=str, default=None, metadata={'json': 'department_name'})
    department_path = attr.ib(type=str, default=None, metadata={'json': 'department_path'})
    total_user_num = attr.ib(type=int, default=None, metadata={'json': 'total_user_num'})
    active_user_num = attr.ib(type=int, default=None, metadata={'json': 'active_user_num'})
    active_user_rate = attr.ib(type=str, default=None, metadata={'json': 'active_user_rate'})
    suite_dau = attr.ib(type=int, default=None, metadata={'json': 'suite_dau'})
    suite_active_rate = attr.ib(type=str, default=None, metadata={'json': 'suite_active_rate'})
    new_user_num = attr.ib(type=int, default=None, metadata={'json': 'new_user_num'})
    new_active_num = attr.ib(type=int, default=None, metadata={'json': 'new_active_num'})
    resign_user_num = attr.ib(type=int, default=None, metadata={'json': 'resign_user_num'})
    im_dau = attr.ib(type=int, default=None, metadata={'json': 'im_dau'})
    send_messenger_user_num = attr.ib(type=int, default=None, metadata={'json': 'send_messenger_user_num'})
    send_messenger_num = attr.ib(type=int, default=None, metadata={'json': 'send_messenger_num'})
    avg_send_messenger_num = attr.ib(type=str, default=None, metadata={'json': 'avg_send_messenger_num'})
    docs_dau = attr.ib(type=int, default=None, metadata={'json': 'docs_dau'})
    create_docs_user_num = attr.ib(type=int, default=None, metadata={'json': 'create_docs_user_num'})
    create_docs_num = attr.ib(type=int, default=None, metadata={'json': 'create_docs_num'})
    avg_create_docs_num = attr.ib(type=str, default=None, metadata={'json': 'avg_create_docs_num'})
    cal_dau = attr.ib(type=int, default=None, metadata={'json': 'cal_dau'})
    create_cal_user_num = attr.ib(type=int, default=None, metadata={'json': 'create_cal_user_num'})
    create_cal_num = attr.ib(type=int, default=None, metadata={'json': 'create_cal_num'})
    avg_create_cal_num = attr.ib(type=str, default=None, metadata={'json': 'avg_create_cal_num'})
    vc_dau = attr.ib(type=int, default=None, metadata={'json': 'vc_dau'})
    vc_duration = attr.ib(type=int, default=None, metadata={'json': 'vc_duration'})
    avg_vc_duration = attr.ib(type=str, default=None, metadata={'json': 'avg_vc_duration'})


@to_json_decorator
@attr.s
class AdminUserStat(object):
    date = attr.ib(type=str, default=None, metadata={'json': 'date'})
    user_id = attr.ib(type=str, default=None, metadata={'json': 'user_id'})
    user_name = attr.ib(type=str, default=None, metadata={'json': 'user_name'})
    department_name = attr.ib(type=str, default=None, metadata={'json': 'department_name'})
    department_path = attr.ib(type=str, default=None, metadata={'json': 'department_path'})
    create_time = attr.ib(type=str, default=None, metadata={'json': 'create_time'})
    user_active_flag = attr.ib(type=int, default=None, metadata={'json': 'user_active_flag'})
    register_time = attr.ib(type=str, default=None, metadata={'json': 'register_time'})
    suite_active_flag = attr.ib(type=int, default=None, metadata={'json': 'suite_active_flag'})
    last_active_time = attr.ib(type=str, default=None, metadata={'json': 'last_active_time'})
    im_active_flag = attr.ib(type=int, default=None, metadata={'json': 'im_active_flag'})
    send_messenger_num = attr.ib(type=int, default=None, metadata={'json': 'send_messenger_num'})
    docs_active_flag = attr.ib(type=int, default=None, metadata={'json': 'docs_active_flag'})
    create_docs_num = attr.ib(type=int, default=None, metadata={'json': 'create_docs_num'})
    cal_active_flag = attr.ib(type=int, default=None, metadata={'json': 'cal_active_flag'})
    create_cal_num = attr.ib(type=int, default=None, metadata={'json': 'create_cal_num'})
    vc_active_flag = attr.ib(type=int, default=None, metadata={'json': 'vc_active_flag'})
    vc_duration = attr.ib(type=int, default=None, metadata={'json': 'vc_duration'})
    active_os = attr.ib(type=str, default=None, metadata={'json': 'active_os'})


@to_json_decorator
@attr.s
class SplashPageStat(object):
    date = attr.ib(type=str, default=None, metadata={'json': 'date'})
    splash_id = attr.ib(type=str, default=None, metadata={'json': 'splash_id'})
    impression_count = attr.ib(type=int, default=None, metadata={'json': 'impression_count'})
    click_count = attr.ib(type=int, default=None, metadata={'json': 'click_count'})
    skip_count = attr.ib(type=int, default=None, metadata={'json': 'skip_count'})
    impression_count_accumulate = attr.ib(type=int, default=None, metadata={'json': 'impression_count_accumulate'})
    click_count_accumulate = attr.ib(type=int, default=None, metadata={'json': 'click_count_accumulate'})
    skip_count_accumulate = attr.ib(type=int, default=None, metadata={'json': 'skip_count_accumulate'})




@attr.s
class AdminDeptStatListResult(object):
    has_more = attr.ib(type=bool, default=None, metadata={'json': 'has_more'})
    page_token = attr.ib(type=str, default=None, metadata={'json': 'page_token'})
    items = attr.ib(type=List[AdminDeptStat], default=None, metadata={'json': 'items'})



@attr.s
class AdminUserStatListResult(object):
    has_more = attr.ib(type=bool, default=None, metadata={'json': 'has_more'})
    page_token = attr.ib(type=str, default=None, metadata={'json': 'page_token'})
    items = attr.ib(type=List[AdminUserStat], default=None, metadata={'json': 'items'})


# -*- coding: UTF-8 -*-
# Code generated by lark suite oapi sdk gen

from typing import List, Dict, Any
from ....utils.dt import to_json_decorator
import attr




@to_json_decorator
@attr.s
class FileConfig(object):
    file_id = attr.ib(type=str, default=None, metadata={'json': 'file_id'})
    format = attr.ib(type=str, default=None, metadata={'json': 'format'})
    engine_type = attr.ib(type=str, default=None, metadata={'json': 'engine_type'})


@to_json_decorator
@attr.s
class StreamConfig(object):
    stream_id = attr.ib(type=str, default=None, metadata={'json': 'stream_id'})
    sequence_id = attr.ib(type=int, default=None, metadata={'json': 'sequence_id'})
    action = attr.ib(type=int, default=None, metadata={'json': 'action'})
    format = attr.ib(type=str, default=None, metadata={'json': 'format'})
    engine_type = attr.ib(type=str, default=None, metadata={'json': 'engine_type'})


@to_json_decorator
@attr.s
class Speech(object):
    speech = attr.ib(type=str, default=None, metadata={'json': 'speech'})
    speech_key = attr.ib(type=str, default=None, metadata={'json': 'speech_key'})



@to_json_decorator
@attr.s
class SpeechFileRecognizeReqBody(object):
    speech = attr.ib(type=Speech, default=None, metadata={'json': 'speech'})
    config = attr.ib(type=FileConfig, default=None, metadata={'json': 'config'})


@attr.s
class SpeechFileRecognizeResult(object):
    recognition_text = attr.ib(type=str, default=None, metadata={'json': 'recognition_text'})


@to_json_decorator
@attr.s
class SpeechStreamRecognizeReqBody(object):
    speech = attr.ib(type=Speech, default=None, metadata={'json': 'speech'})
    config = attr.ib(type=StreamConfig, default=None, metadata={'json': 'config'})


@attr.s
class SpeechStreamRecognizeResult(object):
    stream_id = attr.ib(type=str, default=None, metadata={'json': 'stream_id'})
    sequence_id = attr.ib(type=int, default=None, metadata={'json': 'sequence_id'})
    recognition_text = attr.ib(type=str, default=None, metadata={'json': 'recognition_text'})


# -*- coding: UTF-8 -*-

class AppSettings(object):
    """
    This class is to storage basic app_type, app_id, app_secret for application.
    """

    def __init__(self, app_type, app_id, app_secret, verification_token, encrypt_key):
        # type: (str, str, str, str, str) -> None
        self.app_type = app_type
        self.app_id = app_id
        self.app_secret = app_secret
        self.verification_token = verification_token
        self.encrypt_key = encrypt_key



# -*- coding: UTF-8 -*-
# Code generated by lark suite oapi sdk gen

from typing import List, Dict, Any
from ....utils.dt import to_json_decorator
import attr




@to_json_decorator
@attr.s
class User(object):
    member_type = attr.ib(type=str, default=None, metadata={'json': 'memberType'})
    member_id = attr.ib(type=str, default=None, metadata={'json': 'memberId'})


@to_json_decorator
@attr.s
class Editors(object):
    users = attr.ib(type=List[User], default=None, metadata={'json': 'users'})
    add_editors = attr.ib(type=List[User], default=None, metadata={'json': 'addEditors'})
    del_editors = attr.ib(type=List[User], default=None, metadata={'json': 'delEditors'})


@to_json_decorator
@attr.s
class Source(object):
    sheet_id = attr.ib(type=str, default=None, metadata={'json': 'sheetId'})


@to_json_decorator
@attr.s
class Protect(object):
    lock = attr.ib(type=str, default=None, metadata={'json': 'lock'})
    lock_info = attr.ib(type=str, default=None, metadata={'json': 'lockInfo'})
    user_ids = attr.ib(type=List[int], default=None, metadata={'json': 'userIds'})


@to_json_decorator
@attr.s
class Properties(object):
    sheet_id = attr.ib(type=str, default=None, metadata={'json': 'sheetId'})
    title = attr.ib(type=str, default=None, metadata={'json': 'title'})
    index = attr.ib(type=int, default=None, metadata={'json': 'index'})
    hidden = attr.ib(type=bool, default=None, metadata={'json': 'hidden'})
    frozen_row_count = attr.ib(type=int, default=None, metadata={'json': 'frozenRowCount'})
    frozen_col_count = attr.ib(type=int, default=None, metadata={'json': 'frozenColCount'})
    protect = attr.ib(type=Protect, default=None, metadata={'json': 'protect'})


@to_json_decorator
@attr.s
class AddSheet(object):
    properties = attr.ib(type=Properties, default=None, metadata={'json': 'properties'})


@to_json_decorator
@attr.s
class OptionsResponse(object):
    multiple_values = attr.ib(type=bool, default=None, metadata={'json': 'multipleValues'})
    highlight_valid_data = attr.ib(type=bool, default=None, metadata={'json': 'highlightValidData'})
    color_value_map = attr.ib(type=Dict[str, str], default=None, metadata={'json': 'colorValueMap'})


@to_json_decorator
@attr.s
class DataValidationResponse(object):
    condition_values = attr.ib(type=List[str], default=None, metadata={'json': 'conditionValues'})
    data_validation_id = attr.ib(type=int, default=None, metadata={'json': 'dataValidationId'})
    data_validation_type = attr.ib(type=str, default=None, metadata={'json': 'dataValidationType'})
    options = attr.ib(type=OptionsResponse, default=None, metadata={'json': 'options'})


@to_json_decorator
@attr.s
class Options(object):
    multiple_values = attr.ib(type=bool, default=None, metadata={'json': 'multipleValues'})
    highlight_valid_data = attr.ib(type=bool, default=None, metadata={'json': 'highlightValidData'})
    colors = attr.ib(type=List[str], default=None, metadata={'json': 'colors'})


@to_json_decorator
@attr.s
class DataValidationRequest(object):
    condition_values = attr.ib(type=List[str], default=None, metadata={'json': 'conditionValues'})
    data_validation_id = attr.ib(type=int, default=None, metadata={'json': 'dataValidationId'})
    data_validation_type = attr.ib(type=str, default=None, metadata={'json': 'dataValidationType'})
    options = attr.ib(type=Options, default=None, metadata={'json': 'options'})


@to_json_decorator
@attr.s
class Merge(object):
    start_row_index = attr.ib(type=int, default=None, metadata={'json': 'startRowIndex'})
    start_column_index = attr.ib(type=int, default=None, metadata={'json': 'startColumnIndex'})
    row_count = attr.ib(type=int, default=None, metadata={'json': 'rowCount'})
    column_count = attr.ib(type=int, default=None, metadata={'json': 'columnCount'})


@to_json_decorator
@attr.s
class Font(object):
    bold = attr.ib(type=bool, default=None, metadata={'json': 'bold'})
    italic = attr.ib(type=bool, default=None, metadata={'json': 'italic'})
    font_size = attr.ib(type=str, default=None, metadata={'json': 'fontSize'})
    clean = attr.ib(type=bool, default=None, metadata={'json': 'clean'})


@to_json_decorator
@attr.s
class Style(object):
    font = attr.ib(type=Font, default=None, metadata={'json': 'font'})
    text_decoration = attr.ib(type=int, default=None, metadata={'json': 'textDecoration'})
    formatter = attr.ib(type=str, default=None, metadata={'json': 'formatter'})
    h_align = attr.ib(type=int, default=None, metadata={'json': 'hAlign'})
    v_align = attr.ib(type=int, default=None, metadata={'json': 'vAlign'})
    fore_color = attr.ib(type=str, default=None, metadata={'json': 'foreColor'})
    back_color = attr.ib(type=str, default=None, metadata={'json': 'backColor'})
    border_type = attr.ib(type=str, default=None, metadata={'json': 'borderType'})
    border_color = attr.ib(type=str, default=None, metadata={'json': 'borderColor'})
    clean = attr.ib(type=bool, default=None, metadata={'json': 'clean'})


@to_json_decorator
@attr.s
class AppendStyleBatch(object):
    ranges = attr.ib(type=List[str], default=None, metadata={'json': 'ranges'})
    style = attr.ib(type=Style, default=None, metadata={'json': 'style'})


@to_json_decorator
@attr.s
class AppendStyle(object):
    range = attr.ib(type=str, default=None, metadata={'json': 'range'})
    style = attr.ib(type=Style, default=None, metadata={'json': 'style'})


@to_json_decorator
@attr.s
class Dimension(object):
    sheet_id = attr.ib(type=str, default=None, metadata={'json': 'sheetId'})
    major_dimension = attr.ib(type=str, default=None, metadata={'json': 'majorDimension'})
    start_index = attr.ib(type=int, default=None, metadata={'json': 'startIndex'})
    end_index = attr.ib(type=int, default=None, metadata={'json': 'endIndex'})
    length = attr.ib(type=int, default=None, metadata={'json': 'length'})


@to_json_decorator
@attr.s
class ProtectedRangeUpdate(object):
    protect_id = attr.ib(type=str, default=None, metadata={'json': 'protectId'})
    dimension = attr.ib(type=Dimension, default=None, metadata={'json': 'dimension'})
    sheet_id = attr.ib(type=str, default=None, metadata={'json': 'sheetId'})
    lock_info = attr.ib(type=str, default=None, metadata={'json': 'lockInfo'})
    editors = attr.ib(type=Editors, default=None, metadata={'json': 'editors'})


@to_json_decorator
@attr.s
class ProtectedRange(object):
    protect_id = attr.ib(type=str, default=None, metadata={'json': 'protectId'})
    dimension = attr.ib(type=Dimension, default=None, metadata={'json': 'dimension'})
    sheet_id = attr.ib(type=str, default=None, metadata={'json': 'sheetId'})
    lock_info = attr.ib(type=str, default=None, metadata={'json': 'lockInfo'})
    editors = attr.ib(type=Editors, default=None, metadata={'json': 'editors'})


@to_json_decorator
@attr.s
class Sheet(object):
    sheet_id = attr.ib(type=str, default=None, metadata={'json': 'sheetId'})
    title = attr.ib(type=str, default=None, metadata={'json': 'title'})
    index = attr.ib(type=int, default=None, metadata={'json': 'index'})
    row_count = attr.ib(type=int, default=None, metadata={'json': 'rowCount'})
    column_count = attr.ib(type=int, default=None, metadata={'json': 'columnCount'})
    frozen_row_count = attr.ib(type=int, default=None, metadata={'json': 'frozenRowCount'})
    frozen_col_count = attr.ib(type=int, default=None, metadata={'json': 'frozenColCount'})
    merges = attr.ib(type=List[Merge], default=None, metadata={'json': 'merges'})
    protected_range = attr.ib(type=ProtectedRange, default=None, metadata={'json': 'protectedRange'})


@to_json_decorator
@attr.s
class AddProtectedDimension(object):
    dimension = attr.ib(type=Dimension, default=None, metadata={'json': 'dimension'})
    editors = attr.ib(type=List[int], default=None, metadata={'json': 'editors'})
    lock_info = attr.ib(type=str, default=None, metadata={'json': 'lockInfo'})
    protect_id = attr.ib(type=str, default=None, metadata={'json': 'protectId'})


@to_json_decorator
@attr.s
class Destination(object):
    title = attr.ib(type=str, default=None, metadata={'json': 'title'})


@to_json_decorator
@attr.s
class CopySheet(object):
    source = attr.ib(type=Source, default=None, metadata={'json': 'source'})
    destination = attr.ib(type=Destination, default=None, metadata={'json': 'destination'})


@to_json_decorator
@attr.s
class DeleteSheetReply(object):
    result = attr.ib(type=bool, default=None, metadata={'json': 'result'})
    sheet_id = attr.ib(type=str, default=None, metadata={'json': 'sheetId'})


@to_json_decorator
@attr.s
class Reply(object):
    update_sheet = attr.ib(type=Properties, default=None, metadata={'json': 'updateSheet'})
    add_sheet = attr.ib(type=Properties, default=None, metadata={'json': 'addSheet'})
    copy_sheet = attr.ib(type=Properties, default=None, metadata={'json': 'copySheet'})
    delete_sheet = attr.ib(type=DeleteSheetReply, default=None, metadata={'json': 'deleteSheet'})


@to_json_decorator
@attr.s
class DeleteSheet(object):
    sheet_id = attr.ib(type=str, default=None, metadata={'json': 'sheetId'})


@to_json_decorator
@attr.s
class Requests(object):
    add_sheet = attr.ib(type=AddSheet, default=None, metadata={'json': 'addSheet'})
    copy_sheet = attr.ib(type=CopySheet, default=None, metadata={'json': 'copySheet'})
    delete_sheet = attr.ib(type=DeleteSheet, default=None, metadata={'json': 'deleteSheet'})
    update_sheet = attr.ib(type=Properties, default=None, metadata={'json': 'updateSheet'})


@to_json_decorator
@attr.s
class ConditionFormatFont(object):
    bold = attr.ib(type=bool, default=None, metadata={'json': 'bold'})
    italic = attr.ib(type=bool, default=None, metadata={'json': 'italic'})


@to_json_decorator
@attr.s
class ConditionFormatStyle(object):
    font = attr.ib(type=ConditionFormatFont, default=None, metadata={'json': 'font'})
    text_decoration = attr.ib(type=int, default=None, metadata={'json': 'text_decoration'})
    fore_color = attr.ib(type=str, default=None, metadata={'json': 'fore_color'})
    back_color = attr.ib(type=str, default=None, metadata={'json': 'back_color'})


@to_json_decorator
@attr.s
class ConditionFormatAttr(object):
    operator = attr.ib(type=str, default=None, metadata={'json': 'operator'})
    time_period = attr.ib(type=str, default=None, metadata={'json': 'time_period'})
    formula = attr.ib(type=List[str], default=None, metadata={'json': 'formula'})
    text = attr.ib(type=str, default=None, metadata={'json': 'text'})


@to_json_decorator
@attr.s
class ConditionFormat(object):
    cf_id = attr.ib(type=str, default=None, metadata={'json': 'cf_id'})
    ranges = attr.ib(type=List[str], default=None, metadata={'json': 'ranges'})
    rule_type = attr.ib(type=str, default=None, metadata={'json': 'rule_type'})
    attrs = attr.ib(type=List[ConditionFormatAttr], default=None, metadata={'json': 'attrs'})
    style = attr.ib(type=ConditionFormatStyle, default=None, metadata={'json': 'style'})


@to_json_decorator
@attr.s
class SheetConditionFormat(object):
    sheet_id = attr.ib(type=str, default=None, metadata={'json': 'sheet_id'})
    condition_format = attr.ib(type=ConditionFormat, default=None, metadata={'json': 'condition_format'})


@to_json_decorator
@attr.s
class ConditionFormatResponse(object):
    sheet_id = attr.ib(type=str, default=None, metadata={'json': 'sheet_id'})
    cf_id = attr.ib(type=str, default=None, metadata={'json': 'cf_id'})
    res_code = attr.ib(type=int, default=None, metadata={'json': 'res_code'})
    res_msg = attr.ib(type=str, default=None, metadata={'json': 'res_msg'})


@to_json_decorator
@attr.s
class DataValidationDeleteResult(object):
    range = attr.ib(type=str, default=None, metadata={'json': 'range'})
    msg = attr.ib(type=str, default=None, metadata={'json': 'msg'})
    success = attr.ib(type=bool, default=None, metadata={'json': 'success'})
    updated_cells = attr.ib(type=int, default=None, metadata={'json': 'updatedCells'})


@to_json_decorator
@attr.s
class DataValidationRange(object):
    range = attr.ib(type=str, default=None, metadata={'json': 'range'})
    data_validation_ids = attr.ib(type=List[int], default=None, metadata={'json': 'dataValidationIds'})


@to_json_decorator
@attr.s
class DimensionProperties(object):
    visible = attr.ib(type=bool, default=None, metadata={'json': 'visible'})
    fixed_size = attr.ib(type=int, default=None, metadata={'json': 'fixedSize'})


@to_json_decorator
@attr.s
class MetainfoProperties(object):
    title = attr.ib(type=str, default=None, metadata={'json': 'title'})
    owner_user = attr.ib(type=int, default=None, metadata={'json': 'ownerUser'})
    sheet_count = attr.ib(type=int, default=None, metadata={'json': 'sheetCount'})
    revision = attr.ib(type=int, default=None, metadata={'json': 'revision'})


@to_json_decorator
@attr.s
class SheetCfId(object):
    sheet_id = attr.ib(type=str, default=None, metadata={'json': 'sheet_id'})
    cf_id = attr.ib(type=str, default=None, metadata={'json': 'cf_id'})


@to_json_decorator
@attr.s
class SpreadsheetToken(object):
    range = attr.ib(type=str, default=None, metadata={'json': 'range'})


@to_json_decorator
@attr.s
class Spreadsheets(object):
    pass


@to_json_decorator
@attr.s
class UpdateResponse(object):
    spreadsheet_token = attr.ib(type=str, default=None, metadata={'json': 'spreadsheetToken'})
    updated_range = attr.ib(type=str, default=None, metadata={'json': 'updatedRange'})
    updated_rows = attr.ib(type=int, default=None, metadata={'json': 'updatedRows'})
    updated_columns = attr.ib(type=int, default=None, metadata={'json': 'updatedColumns'})
    updated_cells = attr.ib(type=int, default=None, metadata={'json': 'updatedCells'})


@to_json_decorator
@attr.s
class Updates(object):
    spreadsheet_token = attr.ib(type=str, default=None, metadata={'json': 'spreadsheetToken'})
    updated_range = attr.ib(type=str, default=None, metadata={'json': 'updatedRange'})
    updated_rows = attr.ib(type=int, default=None, metadata={'json': 'updatedRows'})
    updated_columns = attr.ib(type=int, default=None, metadata={'json': 'updatedColumns'})
    updated_cells = attr.ib(type=int, default=None, metadata={'json': 'updatedCells'})
    revision = attr.ib(type=int, default=None, metadata={'json': 'revision'})


@to_json_decorator
@attr.s
class ValueRange(object):
    major_dimension = attr.ib(type=str, default=None, metadata={'json': 'majorDimension'})
    range = attr.ib(type=str, default=None, metadata={'json': 'range'})
    revision = attr.ib(type=int, default=None, metadata={'json': 'revision'})
    values = attr.ib(type=List[Any], default=None, metadata={'json': 'values'})



@to_json_decorator
@attr.s
class SpreadsheetsConditionFormatsBatchCreateReqBody(object):
    sheet_condition_formats = attr.ib(type=List[SheetConditionFormat], default=None, metadata={'json': 'sheet_condition_formats'})


@attr.s
class SpreadsheetsConditionFormatsBatchCreateResult(object):
    responses = attr.ib(type=List[ConditionFormatResponse], default=None, metadata={'json': 'responses'})


@to_json_decorator
@attr.s
class SpreadsheetsConditionFormatsBatchDeleteReqBody(object):
    sheet_cf_ids = attr.ib(type=List[SheetCfId], default=None, metadata={'json': 'sheet_cf_ids'})


@attr.s
class SpreadsheetsConditionFormatsBatchDeleteResult(object):
    responses = attr.ib(type=List[ConditionFormatResponse], default=None, metadata={'json': 'responses'})



@attr.s
class SpreadsheetsConditionFormatsBatchGetResult(object):
    sheet_condition_formats = attr.ib(type=List[SheetConditionFormat], default=None, metadata={'json': 'sheet_condition_formats'})


@to_json_decorator
@attr.s
class SpreadsheetsConditionFormatsBatchUpdateReqBody(object):
    sheet_condition_formats = attr.ib(type=List[SheetConditionFormat], default=None, metadata={'json': 'sheet_condition_formats'})


@attr.s
class SpreadsheetsConditionFormatsBatchUpdateResult(object):
    responses = attr.ib(type=List[ConditionFormatResponse], default=None, metadata={'json': 'responses'})


@to_json_decorator
@attr.s
class SpreadsheetsDataValidationCreateReqBody(object):
    range = attr.ib(type=str, default=None, metadata={'json': 'range'})
    data_validation_type = attr.ib(type=str, default=None, metadata={'json': 'dataValidationType'})
    data_validation = attr.ib(type=DataValidationRequest, default=None, metadata={'json': 'dataValidation'})



@to_json_decorator
@attr.s
class SpreadsheetsDataValidationDeleteReqBody(object):
    data_validation_ranges = attr.ib(type=List[DataValidationRange], default=None, metadata={'json': 'dataValidationRanges'})


@attr.s
class SpreadsheetsDataValidationDeleteResult(object):
    range_results = attr.ib(type=List[DataValidationDeleteResult], default=None, metadata={'json': 'rangeResults'})



@attr.s
class SpreadsheetsDataValidationGetResult(object):
    spreadsheet_token = attr.ib(type=str, default=None, metadata={'json': 'spreadsheetToken'})
    sheet_id = attr.ib(type=str, default=None, metadata={'json': 'sheetId'})
    data_validation = attr.ib(type=DataValidationResponse, default=None, metadata={'json': 'dataValidation'})
    revision = attr.ib(type=int, default=None, metadata={'json': 'revision'})


@to_json_decorator
@attr.s
class SpreadsheetsDataValidationUpdateReqBody(object):
    data_validation_type = attr.ib(type=str, default=None, metadata={'json': 'dataValidationType'})
    data_validation = attr.ib(type=DataValidationRequest, default=None, metadata={'json': 'dataValidation'})


@attr.s
class SpreadsheetsDataValidationUpdateResult(object):
    spreadsheet_token = attr.ib(type=str, default=None, metadata={'json': 'spreadsheetToken'})
    sheet_id = attr.ib(type=str, default=None, metadata={'json': 'sheetId'})
    data_validation = attr.ib(type=DataValidationResponse, default=None, metadata={'json': 'dataValidation'})


@to_json_decorator
@attr.s
class SpreadsheetsDimensionRangeAddReqBody(object):
    dimension = attr.ib(type=Dimension, default=None, metadata={'json': 'dimension'})


@attr.s
class SpreadsheetsDimensionRangeAddResult(object):
    add_count = attr.ib(type=int, default=None, metadata={'json': 'addCount'})
    major_dimension = attr.ib(type=str, default=None, metadata={'json': 'majorDimension'})


@to_json_decorator
@attr.s
class SpreadsheetsDimensionRangeDeleteReqBody(object):
    dimension = attr.ib(type=Dimension, default=None, metadata={'json': 'dimension'})


@attr.s
class SpreadsheetsDimensionRangeDeleteResult(object):
    del_count = attr.ib(type=int, default=None, metadata={'json': 'delCount'})
    major_dimension = attr.ib(type=str, default=None, metadata={'json': 'majorDimension'})


@to_json_decorator
@attr.s
class SpreadsheetsDimensionRangeUpdateReqBody(object):
    dimension = attr.ib(type=Dimension, default=None, metadata={'json': 'dimension'})
    dimension_properties = attr.ib(type=DimensionProperties, default=None, metadata={'json': 'dimensionProperties'})



@to_json_decorator
@attr.s
class SpreadsheetsImportReqBody(object):
    file = attr.ib(type=List[int], default=None, metadata={'json': 'file'})
    name = attr.ib(type=str, default=None, metadata={'json': 'name'})
    folder_token = attr.ib(type=str, default=None, metadata={'json': 'folderToken'})


@attr.s
class SpreadsheetsImportResult(object):
    ticket = attr.ib(type=str, default=None, metadata={'json': 'ticket'})



@attr.s
class SpreadsheetsImportResultResult(object):
    ticket = attr.ib(type=str, default=None, metadata={'json': 'ticket'})
    url = attr.ib(type=str, default=None, metadata={'json': 'url'})
    warning_code = attr.ib(type=int, default=None, metadata={'json': 'warningCode'})


@to_json_decorator
@attr.s
class SpreadsheetsInsertDimensionRangeReqBody(object):
    dimension = attr.ib(type=Dimension, default=None, metadata={'json': 'dimension'})
    inherit_style = attr.ib(type=str, default=None, metadata={'json': 'inheritStyle'})



@to_json_decorator
@attr.s
class SpreadsheetsMergeCellsReqBody(object):
    spreadsheet_token = attr.ib(type=str, default=None, metadata={'json': 'spreadsheetToken'})
    range = attr.ib(type=str, default=None, metadata={'json': 'range'})
    merge_type = attr.ib(type=str, default=None, metadata={'json': 'mergeType'})


@attr.s
class SpreadsheetsMergeCellsResult(object):
    spreadsheet_token = attr.ib(type=str, default=None, metadata={'json': 'spreadsheetToken'})



@attr.s
class SpreadsheetsMetainfoResult(object):
    spreadsheet_token = attr.ib(type=str, default=None, metadata={'json': 'spreadsheetToken'})
    properties = attr.ib(type=MetainfoProperties, default=None, metadata={'json': 'properties'})
    sheets = attr.ib(type=List[Sheet], default=None, metadata={'json': 'sheets'})


@to_json_decorator
@attr.s
class SpreadsheetsProtectedRangeBatchCreateReqBody(object):
    add_protected_dimension = attr.ib(type=List[AddProtectedDimension], default=None, metadata={'json': 'addProtectedDimension'})


@attr.s
class SpreadsheetsProtectedRangeBatchCreateResult(object):
    add_protected_dimension = attr.ib(type=List[AddProtectedDimension], default=None, metadata={'json': 'addProtectedDimension'})


@to_json_decorator
@attr.s
class SpreadsheetsProtectedRangeBatchDeleteReqBody(object):
    protect_ids = attr.ib(type=List[str], default=None, metadata={'json': 'protectIds'})


@attr.s
class SpreadsheetsProtectedRangeBatchDeleteResult(object):
    del_protect_ids = attr.ib(type=List[str], default=None, metadata={'json': 'delProtectIds'})



@attr.s
class SpreadsheetsProtectedRangeBatchGetResult(object):
    protected_range = attr.ib(type=List[ProtectedRange], default=None, metadata={'json': 'protectedRange'})


@to_json_decorator
@attr.s
class SpreadsheetsProtectedRangeBatchUpdateReqBody(object):
    requests = attr.ib(type=List[ProtectedRangeUpdate], default=None, metadata={'json': 'requests'})


@attr.s
class SpreadsheetsProtectedRangeBatchUpdateResult(object):
    replies = attr.ib(type=List[ProtectedRangeUpdate], default=None, metadata={'json': 'replies'})


@to_json_decorator
@attr.s
class SpreadsheetsSheetsBatchUpdateReqBody(object):
    requests = attr.ib(type=Requests, default=None, metadata={'json': 'requests'})


@attr.s
class SpreadsheetsSheetsBatchUpdateResult(object):
    replies = attr.ib(type=List[Reply], default=None, metadata={'json': 'replies'})


@to_json_decorator
@attr.s
class SpreadsheetsSheetsUpdatePropertiesReqBody(object):
    properties = attr.ib(type=Properties, default=None, metadata={'json': 'properties'})


@attr.s
class SpreadsheetsSheetsUpdatePropertiesResult(object):
    spreadsheet_token = attr.ib(type=str, default=None, metadata={'json': 'spreadsheetToken'})
    title = attr.ib(type=str, default=None, metadata={'json': 'title'})


@to_json_decorator
@attr.s
class SpreadsheetsStyleUpdateReqBody(object):
    append_style = attr.ib(type=AppendStyle, default=None, metadata={'json': 'appendStyle'})


@attr.s
class SpreadsheetsStyleUpdateResult(object):
    spreadsheet_token = attr.ib(type=str, default=None, metadata={'json': 'spreadsheetToken'})
    updated_range = attr.ib(type=str, default=None, metadata={'json': 'updatedRange'})
    updated_rows = attr.ib(type=int, default=None, metadata={'json': 'updatedRows'})
    updated_columns = attr.ib(type=int, default=None, metadata={'json': 'updatedColumns'})
    updated_cells = attr.ib(type=int, default=None, metadata={'json': 'updatedCells'})
    revision = attr.ib(type=int, default=None, metadata={'json': 'revision'})


@to_json_decorator
@attr.s
class SpreadsheetsStylesBatchUpdateReqBody(object):
    data = attr.ib(type=AppendStyleBatch, default=None, metadata={'json': 'data'})


@attr.s
class SpreadsheetsStylesBatchUpdateResult(object):
    spreadsheet_token = attr.ib(type=str, default=None, metadata={'json': 'spreadsheetToken'})
    total_updated_rows = attr.ib(type=int, default=None, metadata={'json': 'totalUpdatedRows'})
    total_updated_columns = attr.ib(type=int, default=None, metadata={'json': 'totalUpdatedColumns'})
    total_updated_cells = attr.ib(type=int, default=None, metadata={'json': 'totalUpdatedCells'})
    revision = attr.ib(type=int, default=None, metadata={'json': 'revision'})
    responses = attr.ib(type=List[UpdateResponse], default=None, metadata={'json': 'responses'})


@to_json_decorator
@attr.s
class SpreadsheetsUnmergeCellsReqBody(object):
    range = attr.ib(type=str, default=None, metadata={'json': 'range'})


@attr.s
class SpreadsheetsUnmergeCellsResult(object):
    spreadsheet_token = attr.ib(type=str, default=None, metadata={'json': 'spreadsheetToken'})


@to_json_decorator
@attr.s
class SpreadsheetsValuesAppendReqBody(object):
    value_range = attr.ib(type=ValueRange, default=None, metadata={'json': 'valueRange'})


@attr.s
class SpreadsheetsValuesAppendResult(object):
    spreadsheet_token = attr.ib(type=str, default=None, metadata={'json': 'spreadsheetToken'})
    table_range = attr.ib(type=str, default=None, metadata={'json': 'tableRange'})
    revision = attr.ib(type=int, default=None, metadata={'json': 'revision'})
    updates = attr.ib(type=Updates, default=None, metadata={'json': 'updates'})



@attr.s
class SpreadsheetsValuesBatchGetResult(object):
    revision = attr.ib(type=int, default=None, metadata={'json': 'revision'})
    spreadsheet_token = attr.ib(type=str, default=None, metadata={'json': 'spreadsheetToken'})
    total_cells = attr.ib(type=int, default=None, metadata={'json': 'totalCells'})
    value_range = attr.ib(type=List[ValueRange], default=None, metadata={'json': 'valueRange'})


@to_json_decorator
@attr.s
class SpreadsheetsValuesBatchUpdateReqBody(object):
    value_ranges = attr.ib(type=List[ValueRange], default=None, metadata={'json': 'valueRanges'})


@attr.s
class SpreadsheetsValuesBatchUpdateResult(object):
    responses = attr.ib(type=List[UpdateResponse], default=None, metadata={'json': 'responses'})
    revision = attr.ib(type=int, default=None, metadata={'json': 'revision'})
    spreadsheet_token = attr.ib(type=str, default=None, metadata={'json': 'spreadsheetToken'})



@attr.s
class SpreadsheetsValuesGetResult(object):
    revision = attr.ib(type=int, default=None, metadata={'json': 'revision'})
    spreadsheet_token = attr.ib(type=str, default=None, metadata={'json': 'spreadsheetToken'})
    value_range = attr.ib(type=ValueRange, default=None, metadata={'json': 'valueRange'})


@to_json_decorator
@attr.s
class SpreadsheetsValuesImageReqBody(object):
    range = attr.ib(type=str, default=None, metadata={'json': 'range'})
    image = attr.ib(type=List[int], default=None, metadata={'json': 'image'})
    name = attr.ib(type=str, default=None, metadata={'json': 'name'})


@attr.s
class SpreadsheetsValuesImageResult(object):
    spreadsheet_token = attr.ib(type=str, default=None, metadata={'json': 'spreadsheetToken'})


@to_json_decorator
@attr.s
class SpreadsheetsValuesPrependReqBody(object):
    value_range = attr.ib(type=ValueRange, default=None, metadata={'json': 'valueRange'})


@attr.s
class SpreadsheetsValuesPrependResult(object):
    spreadsheet_token = attr.ib(type=str, default=None, metadata={'json': 'spreadsheetToken'})
    table_range = attr.ib(type=str, default=None, metadata={'json': 'tableRange'})
    revision = attr.ib(type=int, default=None, metadata={'json': 'revision'})
    updates = attr.ib(type=Updates, default=None, metadata={'json': 'updates'})


@to_json_decorator
@attr.s
class SpreadsheetsValuesUpdateReqBody(object):
    value_range = attr.ib(type=ValueRange, default=None, metadata={'json': 'valueRange'})


@attr.s
class SpreadsheetsValuesUpdateResult(object):
    spreadsheet_token = attr.ib(type=str, default=None, metadata={'json': 'spreadsheetToken'})
    updated_range = attr.ib(type=str, default=None, metadata={'json': 'updatedRange'})
    updated_rows = attr.ib(type=int, default=None, metadata={'json': 'updatedRows'})
    updated_columns = attr.ib(type=int, default=None, metadata={'json': 'updatedColumns'})
    updated_cells = attr.ib(type=int, default=None, metadata={'json': 'updatedCells'})
    revision = attr.ib(type=int, default=None, metadata={'json': 'revision'})


# -*- coding: UTF-8 -*-
# Code generated by lark suite oapi sdk gen

from typing import List, Dict, Any
from ....utils.dt import to_json_decorator
import attr




@to_json_decorator
@attr.s
class Image(object):
    image_key = attr.ib(type=str, default=None, metadata={'json': 'image_key'})







# -*- coding: UTF-8 -*-
# Code generated by lark suite oapi sdk gen

from typing import Callable

from ....config import Config
from ....context import Context
from ....event.event import set_event_callback

from .model import *


class FileDeletedEventHandler(object):
    def __init__(self, callback):
        # type: (Callable[[Context, Config, FileDeletedEvent], Any]) -> None
        self.handler = callback

    def handle(self, ctx, conf, event):  # type: (Context, Config, FileDeletedEvent) -> Any
        return self.handler(ctx, conf, event)

    @staticmethod
    def set_callback(conf, callback):
        # type: (Config, Callable[[Context, Config, FileDeletedEvent], Any]) -> None
        handler = FileDeletedEventHandler(callback)
        set_event_callback(conf, "drive.file.deleted_v1",
                          handler.handle, clazz=FileDeletedEvent)


class FilePermissionMemberAddedEventHandler(object):
    def __init__(self, callback):
        # type: (Callable[[Context, Config, FilePermissionMemberAddedEvent], Any]) -> None
        self.handler = callback

    def handle(self, ctx, conf, event):  # type: (Context, Config, FilePermissionMemberAddedEvent) -> Any
        return self.handler(ctx, conf, event)

    @staticmethod
    def set_callback(conf, callback):
        # type: (Config, Callable[[Context, Config, FilePermissionMemberAddedEvent], Any]) -> None
        handler = FilePermissionMemberAddedEventHandler(callback)
        set_event_callback(conf, "drive.file.permission_member_added_v1",
                          handler.handle, clazz=FilePermissionMemberAddedEvent)


class FilePermissionMemberRemovedEventHandler(object):
    def __init__(self, callback):
        # type: (Callable[[Context, Config, FilePermissionMemberRemovedEvent], Any]) -> None
        self.handler = callback

    def handle(self, ctx, conf, event):  # type: (Context, Config, FilePermissionMemberRemovedEvent) -> Any
        return self.handler(ctx, conf, event)

    @staticmethod
    def set_callback(conf, callback):
        # type: (Config, Callable[[Context, Config, FilePermissionMemberRemovedEvent], Any]) -> None
        handler = FilePermissionMemberRemovedEventHandler(callback)
        set_event_callback(conf, "drive.file.permission_member_removed_v1",
                          handler.handle, clazz=FilePermissionMemberRemovedEvent)


class FileReadEventHandler(object):
    def __init__(self, callback):
        # type: (Callable[[Context, Config, FileReadEvent], Any]) -> None
        self.handler = callback

    def handle(self, ctx, conf, event):  # type: (Context, Config, FileReadEvent) -> Any
        return self.handler(ctx, conf, event)

    @staticmethod
    def set_callback(conf, callback):
        # type: (Config, Callable[[Context, Config, FileReadEvent], Any]) -> None
        handler = FileReadEventHandler(callback)
        set_event_callback(conf, "drive.file.read_v1",
                          handler.handle, clazz=FileReadEvent)


class FileTitleUpdatedEventHandler(object):
    def __init__(self, callback):
        # type: (Callable[[Context, Config, FileTitleUpdatedEvent], Any]) -> None
        self.handler = callback

    def handle(self, ctx, conf, event):  # type: (Context, Config, FileTitleUpdatedEvent) -> Any
        return self.handler(ctx, conf, event)

    @staticmethod
    def set_callback(conf, callback):
        # type: (Config, Callable[[Context, Config, FileTitleUpdatedEvent], Any]) -> None
        handler = FileTitleUpdatedEventHandler(callback)
        set_event_callback(conf, "drive.file.title_updated_v1",
                          handler.handle, clazz=FileTitleUpdatedEvent)


class FileTrashedEventHandler(object):
    def __init__(self, callback):
        # type: (Callable[[Context, Config, FileTrashedEvent], Any]) -> None
        self.handler = callback

    def handle(self, ctx, conf, event):  # type: (Context, Config, FileTrashedEvent) -> Any
        return self.handler(ctx, conf, event)

    @staticmethod
    def set_callback(conf, callback):
        # type: (Config, Callable[[Context, Config, FileTrashedEvent], Any]) -> None
        handler = FileTrashedEventHandler(callback)
        set_event_callback(conf, "drive.file.trashed_v1",
                          handler.handle, clazz=FileTrashedEvent)



# -*- coding: UTF-8 -*-


from typing import Dict, List, Union

from .oapi_header import OapiHeader


class OapiRequest(object):
    def __init__(self, uri='', body=b'', header=None):
        # type: (str, bytes, Union[Dict[str, Union[str, List[str]]], OapiHeader, None]) -> None
        self.uri = uri
        self.body = body

        if header is None:
            header = {}

        if isinstance(header, dict):
            self.header = OapiHeader(data=header)
        else:
            self.header = header



# -*- coding: UTF-8 -*-
# Code generated by lark suite oapi sdk gen

from typing import List, Dict, Any
from ....utils.dt import to_json_decorator
import attr




@to_json_decorator
@attr.s
class EventLocation(object):
    name = attr.ib(type=str, default=None, metadata={'json': 'name'})
    address = attr.ib(type=str, default=None, metadata={'json': 'address'})
    latitude = attr.ib(type=float, default=None, metadata={'json': 'latitude'})
    longitude = attr.ib(type=float, default=None, metadata={'json': 'longitude'})


@to_json_decorator
@attr.s
class Vchat(object):
    meeting_url = attr.ib(type=str, default=None, metadata={'json': 'meeting_url'})


@to_json_decorator
@attr.s
class TimeInfo(object):
    date = attr.ib(type=str, default=None, metadata={'json': 'date'})
    timestamp = attr.ib(type=str, default=None, metadata={'json': 'timestamp'})
    timezone = attr.ib(type=str, default=None, metadata={'json': 'timezone'})


@to_json_decorator
@attr.s
class EventSearchFilter(object):
    start_time = attr.ib(type=TimeInfo, default=None, metadata={'json': 'start_time'})
    end_time = attr.ib(type=TimeInfo, default=None, metadata={'json': 'end_time'})
    user_ids = attr.ib(type=List[str], default=None, metadata={'json': 'user_ids'})
    room_ids = attr.ib(type=List[str], default=None, metadata={'json': 'room_ids'})
    chat_ids = attr.ib(type=List[str], default=None, metadata={'json': 'chat_ids'})


@to_json_decorator
@attr.s
class Schema(object):
    ui_name = attr.ib(type=str, default=None, metadata={'json': 'ui_name'})
    ui_status = attr.ib(type=str, default=None, metadata={'json': 'ui_status'})
    app_link = attr.ib(type=str, default=None, metadata={'json': 'app_link'})


@to_json_decorator
@attr.s
class Reminder(object):
    minutes = attr.ib(type=int, default=None, metadata={'json': 'minutes'})


@to_json_decorator
@attr.s
class CalendarEvent(object):
    event_id = attr.ib(type=str, default=None, metadata={'json': 'event_id'})
    summary = attr.ib(type=str, default=None, metadata={'json': 'summary'})
    description = attr.ib(type=str, default=None, metadata={'json': 'description'})
    start_time = attr.ib(type=TimeInfo, default=None, metadata={'json': 'start_time'})
    end_time = attr.ib(type=TimeInfo, default=None, metadata={'json': 'end_time'})
    vchat = attr.ib(type=Vchat, default=None, metadata={'json': 'vchat'})
    visibility = attr.ib(type=str, default=None, metadata={'json': 'visibility'})
    attendee_ability = attr.ib(type=str, default=None, metadata={'json': 'attendee_ability'})
    free_busy_status = attr.ib(type=str, default=None, metadata={'json': 'free_busy_status'})
    location = attr.ib(type=EventLocation, default=None, metadata={'json': 'location'})
    color = attr.ib(type=int, default=None, metadata={'json': 'color'})
    reminders = attr.ib(type=List[Reminder], default=None, metadata={'json': 'reminders'})
    recurrence = attr.ib(type=str, default=None, metadata={'json': 'recurrence'})
    status = attr.ib(type=str, default=None, metadata={'json': 'status'})
    is_exception = attr.ib(type=bool, default=None, metadata={'json': 'is_exception'})
    recurring_event_id = attr.ib(type=str, default=None, metadata={'json': 'recurring_event_id'})
    schemas = attr.ib(type=List[Schema], default=None, metadata={'json': 'schemas'})


@to_json_decorator
@attr.s
class AttendeeChatMember(object):
    rsvp_status = attr.ib(type=str, default=None, metadata={'json': 'rsvp_status'})
    is_optional = attr.ib(type=bool, default=None, metadata={'json': 'is_optional'})
    display_name = attr.ib(type=str, default=None, metadata={'json': 'display_name'})
    is_organizer = attr.ib(type=bool, default=None, metadata={'json': 'is_organizer'})
    is_external = attr.ib(type=bool, default=None, metadata={'json': 'is_external'})


@to_json_decorator
@attr.s
class CalendarEventAttendee(object):
    type = attr.ib(type=str, default=None, metadata={'json': 'type'})
    attendee_id = attr.ib(type=str, default=None, metadata={'json': 'attendee_id'})
    rsvp_status = attr.ib(type=str, default=None, metadata={'json': 'rsvp_status'})
    is_optional = attr.ib(type=bool, default=None, metadata={'json': 'is_optional'})
    is_organizer = attr.ib(type=bool, default=None, metadata={'json': 'is_organizer'})
    is_external = attr.ib(type=bool, default=None, metadata={'json': 'is_external'})
    display_name = attr.ib(type=str, default=None, metadata={'json': 'display_name'})
    chat_members = attr.ib(type=List[AttendeeChatMember], default=None, metadata={'json': 'chat_members'})
    user_id = attr.ib(type=str, default=None, metadata={'json': 'user_id'})
    chat_id = attr.ib(type=str, default=None, metadata={'json': 'chat_id'})
    room_id = attr.ib(type=str, default=None, metadata={'json': 'room_id'})
    third_party_email = attr.ib(type=str, default=None, metadata={'json': 'third_party_email'})


@to_json_decorator
@attr.s
class AclScope(object):
    type = attr.ib(type=str, default=None, metadata={'json': 'type'})
    user_id = attr.ib(type=str, default=None, metadata={'json': 'user_id'})


@to_json_decorator
@attr.s
class CalendarAcl(object):
    acl_id = attr.ib(type=str, default=None, metadata={'json': 'acl_id'})
    role = attr.ib(type=str, default=None, metadata={'json': 'role'})
    scope = attr.ib(type=AclScope, default=None, metadata={'json': 'scope'})


@to_json_decorator
@attr.s
class Calendar(object):
    calendar_id = attr.ib(type=str, default=None, metadata={'json': 'calendar_id'})
    summary = attr.ib(type=str, default=None, metadata={'json': 'summary'})
    description = attr.ib(type=str, default=None, metadata={'json': 'description'})
    permissions = attr.ib(type=str, default=None, metadata={'json': 'permissions'})
    color = attr.ib(type=int, default=None, metadata={'json': 'color'})
    type = attr.ib(type=str, default=None, metadata={'json': 'type'})
    summary_alias = attr.ib(type=str, default=None, metadata={'json': 'summary_alias'})
    is_deleted = attr.ib(type=bool, default=None, metadata={'json': 'is_deleted'})
    is_third_party = attr.ib(type=bool, default=None, metadata={'json': 'is_third_party'})
    role = attr.ib(type=str, default=None, metadata={'json': 'role'})


@to_json_decorator
@attr.s
class Freebusy(object):
    start_time = attr.ib(type=str, default=None, metadata={'json': 'start_time'})
    end_time = attr.ib(type=str, default=None, metadata={'json': 'end_time'})


@to_json_decorator
@attr.s
class TimeoffEvent(object):
    timeoff_event_id = attr.ib(type=str, default=None, metadata={'json': 'timeoff_event_id'})
    user_id = attr.ib(type=str, default=None, metadata={'json': 'user_id'})
    timezone = attr.ib(type=str, default=None, metadata={'json': 'timezone'})
    start_time = attr.ib(type=str, default=None, metadata={'json': 'start_time'})
    end_time = attr.ib(type=str, default=None, metadata={'json': 'end_time'})
    title = attr.ib(type=str, default=None, metadata={'json': 'title'})
    description = attr.ib(type=str, default=None, metadata={'json': 'description'})


@to_json_decorator
@attr.s
class Setting(object):
    pass


@to_json_decorator
@attr.s
class CalendarEventAttendeeChatMember(object):
    rsvp_status = attr.ib(type=str, default=None, metadata={'json': 'rsvp_status'})
    is_optional = attr.ib(type=bool, default=None, metadata={'json': 'is_optional'})
    display_name = attr.ib(type=str, default=None, metadata={'json': 'display_name'})
    is_organizer = attr.ib(type=bool, default=None, metadata={'json': 'is_organizer'})
    is_external = attr.ib(type=bool, default=None, metadata={'json': 'is_external'})




@attr.s
class CalendarCreateResult(object):
    calendar = attr.ib(type=Calendar, default=None, metadata={'json': 'calendar'})





@attr.s
class CalendarEventGetResult(object):
    event = attr.ib(type=CalendarEvent, default=None, metadata={'json': 'event'})



@attr.s
class CalendarPatchResult(object):
    calendar = attr.ib(type=Calendar, default=None, metadata={'json': 'calendar'})





@attr.s
class CalendarAclListResult(object):
    acls = attr.ib(type=List[CalendarAcl], default=None, metadata={'json': 'acls'})
    has_more = attr.ib(type=bool, default=None, metadata={'json': 'has_more'})
    page_token = attr.ib(type=str, default=None, metadata={'json': 'page_token'})







@attr.s
class CalendarEventCreateResult(object):
    event = attr.ib(type=CalendarEvent, default=None, metadata={'json': 'event'})



@attr.s
class CalendarListResult(object):
    has_more = attr.ib(type=bool, default=None, metadata={'json': 'has_more'})
    page_token = attr.ib(type=str, default=None, metadata={'json': 'page_token'})
    sync_token = attr.ib(type=str, default=None, metadata={'json': 'sync_token'})
    calendar_list = attr.ib(type=List[Calendar], default=None, metadata={'json': 'calendar_list'})



@attr.s
class CalendarEventAttendeeListResult(object):
    items = attr.ib(type=List[CalendarEventAttendee], default=None, metadata={'json': 'items'})
    has_more = attr.ib(type=bool, default=None, metadata={'json': 'has_more'})
    page_token = attr.ib(type=str, default=None, metadata={'json': 'page_token'})


@to_json_decorator
@attr.s
class CalendarEventAttendeeBatchDeleteReqBody(object):
    attendee_ids = attr.ib(type=List[str], default=None, metadata={'json': 'attendee_ids'})



@to_json_decorator
@attr.s
class CalendarEventAttendeeCreateReqBody(object):
    attendees = attr.ib(type=List[CalendarEventAttendee], default=None, metadata={'json': 'attendees'})


@attr.s
class CalendarEventAttendeeCreateResult(object):
    attendees = attr.ib(type=List[CalendarEventAttendee], default=None, metadata={'json': 'attendees'})





@attr.s
class CalendarEventListResult(object):
    has_more = attr.ib(type=bool, default=None, metadata={'json': 'has_more'})
    page_token = attr.ib(type=str, default=None, metadata={'json': 'page_token'})
    sync_token = attr.ib(type=str, default=None, metadata={'json': 'sync_token'})
    items = attr.ib(type=List[CalendarEvent], default=None, metadata={'json': 'items'})


@to_json_decorator
@attr.s
class CalendarSearchReqBody(object):
    query = attr.ib(type=str, default=None, metadata={'json': 'query'})


@attr.s
class CalendarSearchResult(object):
    items = attr.ib(type=List[Calendar], default=None, metadata={'json': 'items'})
    page_token = attr.ib(type=str, default=None, metadata={'json': 'page_token'})


@to_json_decorator
@attr.s
class FreebusyListReqBody(object):
    time_min = attr.ib(type=str, default=None, metadata={'json': 'time_min'})
    time_max = attr.ib(type=str, default=None, metadata={'json': 'time_max'})
    user_id = attr.ib(type=str, default=None, metadata={'json': 'user_id'})
    room_id = attr.ib(type=str, default=None, metadata={'json': 'room_id'})


@attr.s
class FreebusyListResult(object):
    freebusy_list = attr.ib(type=List[Freebusy], default=None, metadata={'json': 'freebusy_list'})



@attr.s
class CalendarEventPatchResult(object):
    event = attr.ib(type=CalendarEvent, default=None, metadata={'json': 'event'})








@to_json_decorator
@attr.s
class CalendarEventSearchReqBody(object):
    query = attr.ib(type=str, default=None, metadata={'json': 'query'})
    filter = attr.ib(type=EventSearchFilter, default=None, metadata={'json': 'filter'})


@attr.s
class CalendarEventSearchResult(object):
    items = attr.ib(type=List[CalendarEvent], default=None, metadata={'json': 'items'})
    page_token = attr.ib(type=str, default=None, metadata={'json': 'page_token'})



@attr.s
class CalendarSubscribeResult(object):
    calendar = attr.ib(type=Calendar, default=None, metadata={'json': 'calendar'})


@to_json_decorator
@attr.s
class SettingGenerateCaldavConfReqBody(object):
    device_name = attr.ib(type=str, default=None, metadata={'json': 'device_name'})


@attr.s
class SettingGenerateCaldavConfResult(object):
    password = attr.ib(type=str, default=None, metadata={'json': 'password'})
    user_name = attr.ib(type=str, default=None, metadata={'json': 'user_name'})
    server_address = attr.ib(type=str, default=None, metadata={'json': 'server_address'})
    device_name = attr.ib(type=str, default=None, metadata={'json': 'device_name'})









@attr.s
class CalendarEventAttendeeChatMemberListResult(object):
    items = attr.ib(type=List[CalendarEventAttendeeChatMember], default=None, metadata={'json': 'items'})
    has_more = attr.ib(type=bool, default=None, metadata={'json': 'has_more'})
    page_token = attr.ib(type=str, default=None, metadata={'json': 'page_token'})


# -*- coding: UTF-8 -*-


from typing import Union, Any

from .consts import HTTP_HEADER_KEY_REQUEST_ID, HTTP_KEY_STATUS_CODE, HTTP_HEADER_KEY, HTTP_HEADER_KEY_LOG_ID
from .model import OapiHeader


class Context(object):
    def __init__(self):
        # type: () -> None
        self.__data = {}

    def get(self, key):  # type: (str) -> Union[None, Any]
        return self.__data.get(key)

    def set(self, key, value):  # type: (str, Any) -> bool
        self.__data[key] = value
        return True

    def set_request_id(self, log_id, request_id):
        # type: (Union[None, str], Union[None, str]) -> None
        if log_id:
            self.__data[HTTP_HEADER_KEY_REQUEST_ID] = log_id
        else:
            self.__data[HTTP_HEADER_KEY_REQUEST_ID] = request_id

    def get_header(self):  # type: () -> OapiHeader
        header = self.__data[HTTP_HEADER_KEY]
        if header:
            return header
        return OapiHeader()

    def get_request_id(self):  # type: () -> Union[None, str]
        header = self.get_header()
        log_id = header.first_value(HTTP_HEADER_KEY_LOG_ID)
        if log_id:
            return log_id
        return header.first_value(HTTP_HEADER_KEY_REQUEST_ID)

    def get_http_status_code(self):  # type: () -> Union[None, int]
        return self.__data.get(HTTP_KEY_STATUS_CODE)



# -*- coding: UTF-8 -*-

from typing import IO, Any, Callable, Dict, List, Tuple, Union, TypeVar, Generic

import requests

from ...model import OapiHeader
from ... import VERSION
from ...config import Config
from ...consts import *
from ...context import Context
from ...api.api_error import APIError, ERR_ACCESS_TOKEN_TYPE_INVALID, ERR_TENANT_KEY_IS_EMPTY, \
    ERR_USER_ACCESS_TOKEN_KEY_IS_EMPTY, ERR_APP_TICKET_IS_EMPTY
from ..response.response import Response, ResponseError
from ..tokens import *
from .form_data import FormData
from ...consts import ERR_CODE_OK
from ...utils.dt import make_datatype, to_json

T = TypeVar('T')


class Option(object):
    def __init__(self, no_data_field=False, user_access_token='', tenant_key='', is_response_stream=False,
                 path_params=None, query_params=None, timeout=None, response_stream=None):
        # type: (bool, str, str, bool, Union[None, Dict[str, str]], Union[None, Dict[str, str]], Union[None, int], Union[None, IO[Any]]) -> None
        self.no_data_field = no_data_field  # type: bool
        self.user_access_token = user_access_token  # type: str
        self.tenant_key = tenant_key  # type: str
        self.is_response_stream = is_response_stream  # type: bool
        self.path_params = path_params  # type: Union[None, Dict[str, str]]
        self.query_params = query_params  # type: Union[None, Dict[str, str]]
        self.timeout = timeout  # type: Union[None, int]

        # type: Union[None, IO[Any]]
        self.response_stream_file = response_stream


class Request(Generic[T]):

    def __init__(self, http_path, http_method, access_token_types, request_body, output_class=dict, request_opts=None):
        # type: (str, str, Union[List[str], str], Any, T, List[Callable[[Option], Any]]) -> None

        if request_opts:
            # type: List[Callable[[Option], Any]]
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Callable[[Option], Any]]

        self.http_path = http_path
        self.http_method = http_method
        if isinstance(access_token_types, str):
            access_token_types = [access_token_types]

        self.accessible_token_type_list = sorted(access_token_types)

        if self.accessible_token_type_list.count(ACCESS_TOKEN_TYPE_TENANT) > 0:
            self.access_token_type = ACCESS_TOKEN_TYPE_TENANT
        else:
            self.access_token_type = self.accessible_token_type_list[0]

        self.request_body = request_body
        self.output_class = output_class
        self.query_params = {}  # type: Dict[str, str]
        self.response_body = b''  # type: bytes
        self.tenant_key = ''  # type: str
        self.user_access_token = ''  # type: str
        self.content_type = ''  # type: str

        self.is_retry = False  # type: bool
        self.is_response_stream = False  # type: bool
        self.is_response_stream_current = False  # type: bool
        self.response_stream_file = None  # type: Union[None, IO[Any]]
        self.no_data_field = False  # type: bool
        self.timeout = None  # type: Union[None, int]

        self.session = requests.Session()  # type: requests.Session
        self.response = None  # type: Union[requests.Response, None]

    def prepare(self):  # type: () -> None
        option = Option()
        for operation_fn in self.request_opts:
            operation_fn(option)
        self.no_data_field = option.no_data_field
        self.is_response_stream = option.is_response_stream
        self.response_stream_file = option.response_stream_file

        if option.tenant_key != "" \
                and self.accessible_token_type_list.count(ACCESS_TOKEN_TYPE_TENANT):
            self.access_token_type = ACCESS_TOKEN_TYPE_TENANT
            self.tenant_key = option.tenant_key

        if option.user_access_token != "" \
                and self.accessible_token_type_list.count(ACCESS_TOKEN_TYPE_USER):
            self.access_token_type = ACCESS_TOKEN_TYPE_USER
            self.user_access_token = option.user_access_token

        if option.query_params is not None:
            self.query_params = option.query_params

        if option.path_params is not None:
            self.http_path = resolve_path(
                self.http_path, option.path_params)

        if option.timeout is not None:
            self.timeout = option.timeout

    @staticmethod
    def send_by_auth(path, method, body):
        # type: (str, str, Union[bytes, str, FormData]) -> Request
        req = Request(path, method, [ACCESS_TOKEN_TYPE_NONE], body)
        req.request_opts += [set_no_data_field()]
        return req

    def set_authorization_to_header(self, token):  # type: (str) -> None
        self.session.headers['Authorization'] = ('Bearer %s' % token)

    def do_request(self, config):  # type: (Config) -> requests.Response
        method, path = self.http_method, self.http_path
        params, body = self.query_params, self.request_body
        session = self.session

        if isinstance(body, FormData):
            config.logger.debug("request http_path:%s, http_method:%s, access_token_type:%s, request body:%s" % (
                self.http_path, self.http_method, self.access_token_type, body))

            files, data, need_close = body.to_map()
            try:
                self.is_response_stream = True
                self.response = session.request(
                    method, path, files=files, data=data, stream=True, timeout=self.timeout)
            finally:
                for file in need_close:
                    file.close()
        else:
            config.logger.debug("request http_path:%s, http_method:%s, access_token_type:%s, request body:%s" % (
                self.http_path, self.http_method, self.access_token_type, self.request_body))
            if self.content_type:
                session.headers[CONTENT_TYPE] = self.content_type

            if self.response_stream_file:
                self.response = session.request(method, path, params=params, data=body, stream=True,
                                                timeout=self.timeout)
            else:
                self.response = session.request(method, path, params=params, data=body, timeout=self.timeout)

        return self.response

    def prepare_body(self):  # type: () -> None
        if isinstance(self.request_body, FormData):
            for file in self.request_body.files:
                file.prepare()
            return

        body = self.request_body
        if body:
            self.content_type = DEFAULT_CONTENT_TYPE

        if body is None:
            self.request_body = None
        elif isinstance(body, str):
            self.request_body = body.encode('utf-8')
        elif isinstance(body, bytes):
            self.request_body = body
        elif isinstance(body, dict):
            self.request_body = json.dumps(body)
        elif isinstance(body, object):
            self.request_body = json.dumps(to_json(body))
        else:
            raise APIError(message='Unrecognized request_body type')

    def do(self, conf):
        # type: (Config) -> Response[T]
        return Handlers(conf, self).handle()


DEFAULT_MAX_RETRY_COUNT = 1
DEFAULT_HTTP_REQUEST_HEADERS = {
    "User-Agent": "oapi-sdk-python/%s" % VERSION
}


class Handlers(object):
    def __init__(self, config, req, resp=None):
        # type: (Config, Request, Response) -> None
        self.ctx = Context()
        self.config = config
        self.req = req
        self.resp = resp

    def prepare(self):  # type: () -> None
        self.req.prepare()
        req, conf = self.req, self.config

        req.http_path = "%s/%s/%s" % (conf.domain,
                                      OAPI_ROOT_PATH, req.http_path)

    def validate(self):  # type: () -> None
        req, config = self.req, self.config
        if req.access_token_type == ACCESS_TOKEN_TYPE_NONE:
            return

        if not req.accessible_token_type_list.count(req.access_token_type):
            raise ERR_ACCESS_TOKEN_TYPE_INVALID

        if req.access_token_type == ACCESS_TOKEN_TYPE_USER and req.user_access_token == '':
            raise ERR_USER_ACCESS_TOKEN_KEY_IS_EMPTY

        settings = config.app_settings
        if settings.app_type == APP_TYPE_ISV:
            if req.access_token_type == ACCESS_TOKEN_TYPE_TENANT and req.tenant_key == '':
                raise ERR_TENANT_KEY_IS_EMPTY

    def build(self):  # type: () -> None
        req = self.req
        if not req.is_retry:
            req.prepare_body()

        for (k, v) in DEFAULT_HTTP_REQUEST_HEADERS.items():
            req.session.headers[k] = v
        if req.content_type is not None and req.content_type != "":
            req.session.headers[CONTENT_TYPE] = req.content_type

    def sign(self):  # type: () -> None
        req = self.req
        if req.access_token_type == ACCESS_TOKEN_TYPE_APP:
            AccessToken(self).set_app_access_token()
        elif req.access_token_type == ACCESS_TOKEN_TYPE_TENANT:
            AccessToken(self).set_tenant_access_token()
        elif req.access_token_type == ACCESS_TOKEN_TYPE_USER:
            AccessToken(self).set_user_access_token()

    def send(self):  # type: () -> None
        req = self.req
        self.build()
        self.sign()
        req.do_request(self.config)
        ctx, resp = self.ctx, req.response
        ctx.set(HTTP_HEADER_KEY, OapiHeader(resp.headers))
        ctx.set(HTTP_KEY_STATUS_CODE, resp.status_code)

        self.validate_response()
        self.parse_response()
        self.retry()

    def validate_response(self):  # type: () -> None
        conf, req = self.config, self.req
        resp = req.response
        content_type = resp.headers.get(CONTENT_TYPE)
        if req.is_response_stream:
            if content_type.count(CONTENT_TYPE_JSON):
                req.is_response_stream_current = False
                return
            if resp.status_code != 200:
                raise APIError(message="response status_code:%d, not equal 200, body:%s" % (
                    resp.status_code, resp.content))

            req.is_response_stream_current = True
            return

        if not content_type or not content_type.count(CONTENT_TYPE_JSON):
            raise APIError(
                message='response request_id:%s, status_code: %d, content-type: %s, body: %s . if this is a stream, '
                        'please specify response_stream in the constructor function' % (
                            self.ctx.get_request_id(), resp.status_code, content_type, resp.content)
            )

    def parse_response(self):  # type: () -> None
        resp = Response(self.ctx)
        self.resp = resp
        if self.req.is_response_stream_current:
            if self.req.response_stream_file:
                for buffer in self.req.response.iter_content(chunk_size=2048):
                    self.req.response_stream_file.write(buffer)
                resp.data = self.req.response_stream_file
            else:
                resp.data = self.req.response.content
            self.req.response.close()
            return

        resp_json = self.req.response.json()
        self.config.logger.debug("request http_path:%s, response status code:%d, response body:%s" % (
            self.req.http_path, self.req.response.status_code, resp_json))
        if isinstance(resp_json, dict):
            resp.code = resp_json.get('code')
            resp.msg = resp_json.get('msg')
            resp.error = make_datatype(ResponseError, resp_json.get('error'))
            if self.req.no_data_field:
                resp.data = make_datatype(self.req.output_class, resp_json)
            else:
                resp.data = make_datatype(self.req.output_class, resp_json.get('data'))

    def retry(self):  # type: () -> None
        self.req.is_retry = RETRY_CODE.count(self.resp.code) > 0

    def complement(self, exception):  # type: (Exception) -> None
        if isinstance(self.req.request_body, FormData):
            self.req.request_body.delete_tmp_files()

        if exception == ERR_APP_TICKET_IS_EMPTY:
            AccessToken(self).apply_app_ticket()

        if exception:
            return

        if self.resp.code == ERR_CODE_APP_TICKET_INVALID:
            AccessToken(self).apply_app_ticket()

    def handle(self):  # type: () -> Response
        conf, req = self.config, self.req
        exception = None  # type:Exception
        try:
            self.prepare()
            self.validate()
            for _ in range(DEFAULT_MAX_RETRY_COUNT + 1):
                self.send()
                if not req.is_retry:
                    return self.resp
        except Exception as e:
            exception = e
            raise e
        finally:
            self.complement(exception)
        return self.resp


class AccessToken(object):
    def __init__(self, handlers):  # type: (Handlers) -> None
        self.handlers = handlers

    def apply_app_ticket(self):  # type: () -> None
        config = self.handlers.config
        req_body = ApplyAppTicketReq(
            config.app_settings.app_id, config.app_settings.app_secret)
        req = Request.send_by_auth(APPLY_APP_TICKET_PATH, "POST", req_body.__str__())
        resp = Handlers(config, req).handle()
        if resp.code == ERR_CODE_OK:
            config.logger.info("Already applied app ticket")
        else:
            config.logger.error(resp.__str__())

    def set_app_access_token_to_store(self, access_token):
        # type: (AppAccessToken) -> None

        token, expire = access_token.app_access_token, access_token.expire
        conf = self.handlers.config
        key = '%s-%s' % (APP_ACCESS_TOKEN_KEY_PREFIX, conf.app_settings.app_id)
        try:
            conf.store.set(key, token, expire - EXPIRE_DELTA)
        except Exception as e:
            conf.logger.warn('storage app_access_token into storage occurred an error:%s' % e.__str__())

    def set_tenant_access_token_to_store(self, access_token):
        # type: (TenantAccessToken) -> None

        req = self.handlers.req
        token, expire = access_token.tenant_access_token, access_token.expire
        tenant_key = req.tenant_key
        conf = self.handlers.config
        key = '%s-%s-%s' % (TENANT_ACCESS_TOKEN_KEY_PREFIX,
                            conf.app_settings.app_id, tenant_key)
        try:
            conf.store.set(key, token, expire - EXPIRE_DELTA)
        except Exception as e:
            conf.logger.warn(
                'storage tenant_access_token into storage occurred an error:%s' % e.__str__())

    def get_internal_app_access_token(self):  # type: () -> AppAccessToken
        conf = self.handlers.config
        internal_access_token_request = InternalAccessTokenReq(
            conf.app_settings.app_id, conf.app_settings.app_secret)
        req_json = internal_access_token_request.__str__()
        req = Request.send_by_auth(
            APP_ACCESS_TOKEN_INTERNAL_URL_PATH, 'POST', req_json)
        resp = Handlers(conf, req).handle()
        if resp.code != ERR_CODE_OK:
            raise APIError("obtain internal app access token，failure information:%s" % resp)
        return AppAccessToken(resp.data['app_access_token'], resp.data['expire'])

    def get_isv_app_access_token(self):
        # type: () -> AppAccessToken
        find, app_ticket = self.get_app_ticket()
        if not find:
            raise ERR_APP_TICKET_IS_EMPTY

        conf = self.handlers.config
        settings = conf.app_settings

        isv_app_access_token_req = ISVAppAccessTokenReq(
            settings.app_id, settings.app_secret, app_ticket)
        req_json = isv_app_access_token_req.__str__()

        req = Request.send_by_auth(
            APP_ACCESS_TOKEN_ISV_URL_PATH, 'POST', req_json)
        resp = Handlers(conf, req).handle()
        if resp.code != ERR_CODE_OK:
            raise APIError("obtain isv app access token，failure information:%s" % resp)
        return AppAccessToken(resp.data['app_access_token'], resp.data['expire'])

    def get_isv_tenant_access_token(self):
        # type: () -> Tuple[AppAccessToken, TenantAccessToken]
        app_access_token = self.get_isv_app_access_token()
        tenant_key = self.handlers.req.tenant_key

        isv_tenant_access_token_req = ISVTenantAccessTokenReq(
            app_access_token.app_access_token, tenant_key)
        req_json = isv_tenant_access_token_req.__str__()

        req = Request.send_by_auth(
            TENANT_ACCESS_TOKEN_ISV_URL_PATH, 'POST', req_json)
        resp = Handlers(self.handlers.config, req).handle()
        if resp.code != ERR_CODE_OK:
            raise APIError("obtain isv tenant access token，failure information:%s" % resp)
        tenant_access_token = TenantAccessToken(resp.data['tenant_access_token'], resp.data['expire'])
        return app_access_token, tenant_access_token

    def set_app_access_token(self):  # type: () -> None
        conf, req = self.handlers.config, self.handlers.req
        if not req.is_retry:
            key = '%s-%s' % (APP_ACCESS_TOKEN_KEY_PREFIX, conf.app_settings.app_id)
            find, token = conf.store.get(key)
            if find and token != "":
                req.set_authorization_to_header(token)
                return

        if conf.app_settings.app_type == APP_TYPE_INTERNAL:
            app_access_token = self.get_internal_app_access_token()
        else:  # isv app access token
            app_access_token = self.get_isv_app_access_token()

        self.set_app_access_token_to_store(app_access_token)
        req.set_authorization_to_header(app_access_token.app_access_token)

    def set_user_access_token(self):  # type: () -> None
        req = self.handlers.req
        if req.user_access_token != "":
            req.set_authorization_to_header(req.user_access_token)

    def set_tenant_access_token(self):  # type: () -> None
        conf, req = self.handlers.config, self.handlers.req
        tenant_key = req.tenant_key
        if not req.is_retry:
            key = '%s-%s-%s' % (TENANT_ACCESS_TOKEN_KEY_PREFIX,
                                conf.app_settings.app_id, tenant_key)
            find, token = conf.store.get(key)
            if find and token != "":
                req.set_authorization_to_header(token)
                return

        if conf.app_settings.app_type == APP_TYPE_INTERNAL:
            tenant_access_token = self.get_internal_tenant_access_token()
        else:  # isv app access token
            app_access_token, tenant_access_token = self.get_isv_tenant_access_token()
            if app_access_token is not None:
                self.set_app_access_token_to_store(app_access_token)

        self.set_tenant_access_token_to_store(tenant_access_token)
        req.set_authorization_to_header(
            tenant_access_token.tenant_access_token)

    def get_app_ticket(self):  # type: () -> Tuple[bool, str]
        config = self.handlers.config
        store = config.store

        key = '%s-%s' % (APP_TICKET_KEY_PREFIX, config.app_settings.app_id)
        return store.get(key)

    def get_internal_tenant_access_token(self):
        # type: () -> TenantAccessToken
        conf = self.handlers.config
        settings = conf.app_settings

        internal_access_token_req = InternalAccessTokenReq(
            settings.app_id, settings.app_secret)
        req_json = internal_access_token_req.__str__()

        req = Request.send_by_auth(
            TENANT_ACCESS_TOKEN_INTERNAL_URL_PATH, 'POST', req_json)
        resp = Handlers(self.handlers.config, req).handle()
        if resp.code != ERR_CODE_OK:
            raise APIError("obtain internal tenant access token，failure information:%s" % resp)
        return TenantAccessToken(resp.data['tenant_access_token'], resp.data['expire'])


def set_timeout(timeout):
    # type: (Union[int, float]) -> Callable[[Option], Any]
    """
    timeout unit: seconds
    """

    def fn(option):  # type: (Option) -> None
        option.timeout = timeout

    return fn


def set_user_access_token(user_access_token):
    # type: (str) -> Callable[[Option], Any]

    def fn(option):  # type: (Option) -> None
        option.user_access_token = user_access_token

    return fn


def set_tenant_key(tenant_key):
    # type: (str) -> Callable[[Option], Any]

    def fn(option):  # type: (Option) -> None
        option.tenant_key = tenant_key

    return fn


def set_query_params(query_params):
    # type: (Dict[str, str]) -> Callable[[Option], Any]

    def fn(option):  # type: (Option) -> None
        option.query_params = query_params

    return fn


def set_path_params(path_params):
    # type: (Dict[str, str]) -> Callable[[Option], Any]

    def fn(option):  # type: (Option) -> None
        option.path_params = path_params

    return fn


def set_no_data_field():
    # type: () -> Callable[[Option], Any]

    def fn(option):  # type: (Option) -> None
        option.no_data_field = True

    return fn


def set_is_response_stream():
    # type: () -> Callable[[Option], Any]

    def fn(option):  # type: (Option) -> None
        option.is_response_stream = True

    return fn


def set_response_stream(stream):
    # type: (IO[Any]) -> Callable[[Option], Any]

    def fn(option):  # type: (Option) -> None
        option.is_response_stream = True
        option.response_stream_file = stream

    return fn


def resolve_path(path, path_variables):  # type: (str, Dict[str, str]) -> str
    splitted = path.split('/')
    for i in range(len(splitted)):
        if splitted[i].startswith(':'):
            k = splitted[i][1:]
            v = path_variables.get(splitted[i][1:], '')
            if v == '':
                raise RuntimeError("http path:%s, param name `%s` not found value" % (path, k))
            splitted[i] = v
    return '/'.join(splitted)



# -*- coding: UTF-8 -*-


from typing import Any, Callable, Union, Dict

from ..config import Config
from ..context import Context
from ..model.oapi_request import OapiRequest
from ..model.oapi_response import OapiResponse
from .handler.handlers import CardHandlers
from .handler.manager import Manager
from .model.card import HTTPCard, Card


def set_card_callback(conf, callback):
    # type: (Config, Callable[[Context, Config, Card], Union[None, Dict]]) -> None
    Manager.set_callback(conf, callback)


def handle_card(conf, req):
    # type: (Config, OapiRequest) -> OapiResponse
    http_card = HTTPCard()
    http_card.request = req
    http_card.response = OapiResponse()
    CardHandlers(conf, http_card).handler()
    return http_card.response



# -*- coding: UTF-8 -*-


from typing import Dict, ItemsView, List, Union


class OapiHeader(object):
    def __init__(self, data=None):
        # type: (Union[None, Dict[str, List[str]]]) -> None

        self.data = {}  # type: Dict[str, List[str]]
        if data is None:
            self.__append_multiple({})
        else:
            self.__append_multiple(data)

    def __append_multiple(self, headers):
        # type: (Dict[str, Union[str, List[str]]]) -> None
        for k, v in headers.items():
            key = k.lower()

            if self.data.get(key) is None:
                self.data[key] = []

            if not isinstance(v, list):
                v = [v]

            self.data[key] += v

    def append(self, key, value):  # type: (str, Union[str, List[str]]) -> None
        header = {key: value}
        self.__append_multiple(header)

    def get_keys(self):  # type: () -> List[str]
        keys = []
        for key in self.data.keys():
            keys += [key]
        return keys

    def first_value(self, key):  # type: (str) -> Union[None, str]
        val = self.data.get(key.lower())
        if val is None:
            return None

        if isinstance(val, list):
            if len(val) <= 0:
                return None
            else:
                return val[0]

        if isinstance(val, str):
            return val

        return None

    def values(self, key):  # type: (str) -> Union[None, List[str]]
        return self.data.get(key)

    def items(self):  # type: () -> ItemsView[str, List[str]]
        return self.data.items()



# -*- coding: UTF-8 -*-
# Code generated by lark suite oapi sdk gen

from typing import Callable

from ....config import Config
from ....context import Context
from ....event.event import set_event_callback

from .model import *


class MessageReceiveEventHandler(object):
    def __init__(self, callback):
        # type: (Callable[[Context, Config, MessageReceiveEvent], Any]) -> None
        self.handler = callback

    def handle(self, ctx, conf, event):  # type: (Context, Config, MessageReceiveEvent) -> Any
        return self.handler(ctx, conf, event)

    @staticmethod
    def set_callback(conf, callback):
        # type: (Config, Callable[[Context, Config, MessageReceiveEvent], Any]) -> None
        handler = MessageReceiveEventHandler(callback)
        set_event_callback(conf, "im.message.receive_v1",
                          handler.handle, clazz=MessageReceiveEvent)


class ChatUpdatedEventHandler(object):
    def __init__(self, callback):
        # type: (Callable[[Context, Config, ChatUpdatedEvent], Any]) -> None
        self.handler = callback

    def handle(self, ctx, conf, event):  # type: (Context, Config, ChatUpdatedEvent) -> Any
        return self.handler(ctx, conf, event)

    @staticmethod
    def set_callback(conf, callback):
        # type: (Config, Callable[[Context, Config, ChatUpdatedEvent], Any]) -> None
        handler = ChatUpdatedEventHandler(callback)
        set_event_callback(conf, "im.chat.updated_v1",
                          handler.handle, clazz=ChatUpdatedEvent)


class ChatDisbandedEventHandler(object):
    def __init__(self, callback):
        # type: (Callable[[Context, Config, ChatDisbandedEvent], Any]) -> None
        self.handler = callback

    def handle(self, ctx, conf, event):  # type: (Context, Config, ChatDisbandedEvent) -> Any
        return self.handler(ctx, conf, event)

    @staticmethod
    def set_callback(conf, callback):
        # type: (Config, Callable[[Context, Config, ChatDisbandedEvent], Any]) -> None
        handler = ChatDisbandedEventHandler(callback)
        set_event_callback(conf, "im.chat.disbanded_v1",
                          handler.handle, clazz=ChatDisbandedEvent)


class ChatMemberUserAddedEventHandler(object):
    def __init__(self, callback):
        # type: (Callable[[Context, Config, ChatMemberUserAddedEvent], Any]) -> None
        self.handler = callback

    def handle(self, ctx, conf, event):  # type: (Context, Config, ChatMemberUserAddedEvent) -> Any
        return self.handler(ctx, conf, event)

    @staticmethod
    def set_callback(conf, callback):
        # type: (Config, Callable[[Context, Config, ChatMemberUserAddedEvent], Any]) -> None
        handler = ChatMemberUserAddedEventHandler(callback)
        set_event_callback(conf, "im.chat.member.user.added_v1",
                          handler.handle, clazz=ChatMemberUserAddedEvent)


class ChatMemberBotAddedEventHandler(object):
    def __init__(self, callback):
        # type: (Callable[[Context, Config, ChatMemberBotAddedEvent], Any]) -> None
        self.handler = callback

    def handle(self, ctx, conf, event):  # type: (Context, Config, ChatMemberBotAddedEvent) -> Any
        return self.handler(ctx, conf, event)

    @staticmethod
    def set_callback(conf, callback):
        # type: (Config, Callable[[Context, Config, ChatMemberBotAddedEvent], Any]) -> None
        handler = ChatMemberBotAddedEventHandler(callback)
        set_event_callback(conf, "im.chat.member.bot.added_v1",
                          handler.handle, clazz=ChatMemberBotAddedEvent)


class ChatMemberBotDeletedEventHandler(object):
    def __init__(self, callback):
        # type: (Callable[[Context, Config, ChatMemberBotDeletedEvent], Any]) -> None
        self.handler = callback

    def handle(self, ctx, conf, event):  # type: (Context, Config, ChatMemberBotDeletedEvent) -> Any
        return self.handler(ctx, conf, event)

    @staticmethod
    def set_callback(conf, callback):
        # type: (Config, Callable[[Context, Config, ChatMemberBotDeletedEvent], Any]) -> None
        handler = ChatMemberBotDeletedEventHandler(callback)
        set_event_callback(conf, "im.chat.member.bot.deleted_v1",
                          handler.handle, clazz=ChatMemberBotDeletedEvent)


class ChatMemberUserWithdrawnEventHandler(object):
    def __init__(self, callback):
        # type: (Callable[[Context, Config, ChatMemberUserWithdrawnEvent], Any]) -> None
        self.handler = callback

    def handle(self, ctx, conf, event):  # type: (Context, Config, ChatMemberUserWithdrawnEvent) -> Any
        return self.handler(ctx, conf, event)

    @staticmethod
    def set_callback(conf, callback):
        # type: (Config, Callable[[Context, Config, ChatMemberUserWithdrawnEvent], Any]) -> None
        handler = ChatMemberUserWithdrawnEventHandler(callback)
        set_event_callback(conf, "im.chat.member.user.withdrawn_v1",
                          handler.handle, clazz=ChatMemberUserWithdrawnEvent)


class ChatMemberUserDeletedEventHandler(object):
    def __init__(self, callback):
        # type: (Callable[[Context, Config, ChatMemberUserDeletedEvent], Any]) -> None
        self.handler = callback

    def handle(self, ctx, conf, event):  # type: (Context, Config, ChatMemberUserDeletedEvent) -> Any
        return self.handler(ctx, conf, event)

    @staticmethod
    def set_callback(conf, callback):
        # type: (Config, Callable[[Context, Config, ChatMemberUserDeletedEvent], Any]) -> None
        handler = ChatMemberUserDeletedEventHandler(callback)
        set_event_callback(conf, "im.chat.member.user.deleted_v1",
                          handler.handle, clazz=ChatMemberUserDeletedEvent)


class MessageAtMessageReadEventHandler(object):
    def __init__(self, callback):
        # type: (Callable[[Context, Config, MessageAtMessageReadEvent], Any]) -> None
        self.handler = callback

    def handle(self, ctx, conf, event):  # type: (Context, Config, MessageAtMessageReadEvent) -> Any
        return self.handler(ctx, conf, event)

    @staticmethod
    def set_callback(conf, callback):
        # type: (Config, Callable[[Context, Config, MessageAtMessageReadEvent], Any]) -> None
        handler = MessageAtMessageReadEventHandler(callback)
        set_event_callback(conf, "im.message.at_message_read_v1",
                          handler.handle, clazz=MessageAtMessageReadEvent)


class MessageMessageReadEventHandler(object):
    def __init__(self, callback):
        # type: (Callable[[Context, Config, MessageMessageReadEvent], Any]) -> None
        self.handler = callback

    def handle(self, ctx, conf, event):  # type: (Context, Config, MessageMessageReadEvent) -> Any
        return self.handler(ctx, conf, event)

    @staticmethod
    def set_callback(conf, callback):
        # type: (Config, Callable[[Context, Config, MessageMessageReadEvent], Any]) -> None
        handler = MessageMessageReadEventHandler(callback)
        set_event_callback(conf, "im.message.message_read_v1",
                          handler.handle, clazz=MessageMessageReadEvent)
